//
//
//CIVIL AI v9
//
//
//AI for OpenTTD
//© 2018
//
//Turn back, before it's too late.
//


import("pathfinder.road", "RoadPF", 4);
import("Pathfinder.Rail", "RailPF", 1);

class CivilAI extends AIController
{
  function Save();
  function Load(version, data);
  function Start();
  function LoadParas();
  function LocateHomeTown();
  function SetName(HomeTownName);
  function BuildHQ();
  function BuildHomeDepot(hq,hometown);
  function FindHomeDepot();
	
  function BuildDepot(roadtile,town); 
  function IdentifyBus(silent);
  function IdentifyCoach(silent);
  function BuildBuses(town,depot);
  function BuildBusStop(xoff,yoff,count,spread,town,rem,spike);
  function SpikeX(tile,town);
  function SpikeY(tile,town);
  function InterCity();
  function CreateRoute(vehicle,count,stops,depot,routename,fullload,cargo,group);
  function NewNetwork();
  function NewDepot();
  
  function Failure();   
  function MainLoop();
  
  function CashUp();
  function CashDown();
  
  function BusReview();
  function DepotClean();
  function HomeConnect();
  
  function MappaMundi();
  function BuildARoad(a,b,target);
  
  function Vroom();
  function Airportz();  
  function BuildAirport(station,location);
  function BuildSmAp(station,location);
  function BuildComAp(station,location);
  function AirReview();
  function AirAssign(station);
  
  function MyTownList();
  function RoadWork();
  function XingReplace();
  
  function MakeGroups();
  function LoadGroups();
  Groups = [0,0,0,0];
  
 function TruckOps();
 function IdentifyMailTruck(silent);
 function IdentifyGoodsTruck(silent, cargo);
 
 function BuildTruckStop(station,location);
 function NewMailGo(station);
 function NewCargoGo(station, cargo);
 
 function FindCargo(label);
 
 function CargoPlan();
 function MakeAPlan(cargo);
 function SupplyDepot(ind); 
 function BuildSupplyDepot(tile,indloc); 
 
 function ChooChoo();
 function CargoTerminus(cargo);
 function SmallStation(ind, side, orientation, cargo);
 function BuildSmallStation(ind, tile, side, orientation, cargo)
 function TrainPickup(source, ind, cargo);
 function BuildALine(a, b);
 function BuyATrain(a, b, cargo, depot, costing);
 function FindTrainDepot(station);
 function RemoveTracksFrom(station);

  
  // === global vars
  
  HomeDepot = 0;
  IsLoaded = false;
  Dudtowns = AIList();
  NetworkRadius = null;
  MinPop = null;
  MaxBus = null;
  MaxCar = null;
  BuyCar = null;
  MinAirRange = null;
  BuyPlane = null;
  TrainRange = null;
  
  XinglistX =  AIList();
  XinglistY =  AIList(); 

 // === cargo plan elements
  
  PlanDestination = null;
  PlanSource = null;  
  
  DudIndustries = AIList();
  DudTerminus = AIList();
  IndTownList = AIList();
  DudCounter = 0;
  
  Me = (AICompany.ResolveCompanyID(AICompany.COMPANY_SELF));
}
//>
// ====================================================== 
//                       SAVE/LOAD
// ====================================================== 

function CivilAI::Save() {
local Save = {};
return Save;
}  
function CivilAI::Load(version, data) {
IsLoaded = true;
}  


// ====================================================== 
//                     LOAD PARAMETERS
// ====================================================== 

function CivilAI::LoadParas() {
NetworkRadius = AIController.GetSetting("NetworkRadius");
MinPop = AIController.GetSetting("MinPop");
MaxBus = AIController.GetSetting("MaxBus");
MaxCar = AIController.GetSetting("MaxCar");
BuyCar = 10 // (AIController.GetSetting("BuyCar") / 2);
MinAirRange = (AIController.GetSetting("MinAirRange"));
BuyPlane = 2 // (AIController.GetSetting("BuyPlane"));
TrainRange = (AIController.GetSetting("TrainRange"));
}
  
// ====================================================== 
//                         START
// ====================================================== 
  
function CivilAI::Start()
{

// Startup parameters:
AIRoad.SetCurrentRoadType(AIRoad.ROADTYPE_ROAD);
AICompany.SetAutoRenewStatus(false); // we don't do autorenew
 AIRail.SetCurrentRailType(0); // let's assume this is standard rail

 
 

LoadParas();
CashUp();

if (!IsLoaded) {
MakeGroups();
LocateHomeTown();
} else {
LoadGroups();
}

MainLoop();

while (1){
AILog.Info("Never should've come here!");
AIController.Sleep(600);
}
}

// ====================================================== 
//                     COMPANY FAIL
// ====================================================== 

function CivilAI::Failure()
{
AILog.Info("Woe is me!")
AICompany.SetName("Failed CivilAI");
CashDown();
while (1) {
AIController.Sleep(1000000);
}
}

// ====================================================== 
//                      MAIN LOOP
// ====================================================== 

function CivilAI::MainLoop() 
{

if (AIGameSettings.IsDisabledVehicleType(AIVehicle.VT_ROAD)) { // bug out if we can't build buses.
AILog.Info("I rely on building bus networks as the core of my network, but road vehicles are disabled for AI in your settings.");
Failure();
}

if (IsLoaded) {
AILog.Info("Welcome back!");
FindHomeDepot();
} else {

AILog.Info("Now that I have introduced myself, I should like to have some idea of what is going on...");
}

local dosh;
while (true) {

LoadParas();

dosh = AICompany.GetBankBalance(Me);
if (dosh > (AICompany.GetMaxLoanAmount() * 2)) {
CashDown(); // pay off our loan when we're rich, just for the points.
} else {		
CashUp(); 						// Retake loan, just in case inflation is on
}


HomeConnect();					// Check Home Depot
DepotClean();					// Clear Depots
TruckOps();						// Add some trucks
BusReview();					// Bus Review
InterCity();					// Intercity
ChooChoo();						// Build and maintain Trains
CargoPlan();					// Find a goods/food source
NewNetwork();					// New Bus Network
AirReview();
Vroom();						// Town Cars
Airportz();						// Build Airports
XingReplace();					// check for crashes and lost vehicles
MappaMundi();					// Roads - this is also when we tend to build up cash, so put the most important things at the top

}
}

// ====================================================== 
//                    LOAN MANAGEMENT
// ====================================================== 

function CivilAI::CashUp() {

AICompany.SetLoanAmount(AICompany.GetMaxLoanAmount());
//AILog.Info("I've borrowed some money from the bank.")
return
}

function CivilAI::CashDown() {


local b = 0; // just in case
while ((AICompany.GetBankBalance(Me) > AICompany.GetLoanInterval()) && b < 100) {
AICompany.SetLoanAmount(AICompany.GetLoanAmount()-AICompany.GetLoanInterval())
b++
}

//AILog.Info("I've paid off as much of my loan as I can.")
return
}


// ====================================================== 
//                   HOME TOWN SET UP
// ====================================================== 

function CivilAI::LocateHomeTown()
{
AILog.Info("Oh, hello...");

// find a town
local bigtown = null;
local townlist = AITownList();
townlist.Valuate(AITown.GetPopulation);
townlist.RemoveBelowValue(MinPop);

// remove unreachables
foreach (town,z in townlist) {
local towntile = AITown.GetLocation(town);
if (!AITile.HasTransportType(towntile,AITile.TRANSPORT_ROAD)) {
AILog.Info(AITown.GetName(town) + " has no central road tile.");
townlist.RemoveItem(town);
} else {
}
}

townlist.Valuate(AIBase.RandItem); // shuffle the list

foreach (town,z in townlist) {
if (!((AITown.GetLastMonthSupplied(town, 0) * 15) > AITown.GetPopulation(town))) {
bigtown = town;
break;
}
}
	
if (bigtown == null) {	
// we'll do our best...
local townlist = AITownList();
townlist.Valuate(AITown.GetPopulation);
bigtown = townlist.Begin();
}	

//AILog.Info("First, I should probably name the company, and myself...");	
SetName(AITown.GetName(bigtown));
AILog.Info("I'll set up my headquarters in " + (AITown.GetName(bigtown)) + ".");	
BuildHQ(bigtown);
return
}

function CivilAI::SetName(HomeTownName)
{
local firstnamem = [0, 1, 2, 3, 4, 5, 6, 7]
local firstnamef = [0, 1, 2, 3, 4, 5, 6, 7]
local lastname = [0, 1, 2, 3, 4, 5, 6, 7]

firstnamem[0] = ("Rudolph");
firstnamem[1] = ("Gordon");
firstnamem[2] = ("Elvis");
firstnamem[3] = ("Pedro");
firstnamem[4] = ("Hubert");
firstnamem[5] = ("Alan");
firstnamem[6] = ("Edgar");
firstnamem[7] = ("Tony");

firstnamef[0] = ("Florence");
firstnamef[1] = ("Madame");
firstnamef[2] = ("Delia");
firstnamef[3] = ("Gertrude");
firstnamef[4] = ("Virginia");
firstnamef[5] = ("Madge");
firstnamef[6] = ("Anita");
firstnamef[7] = ("Juliet");

lastname[0] = ("Wideload");
lastname[1] = ("McBus");
lastname[2] = ("Honker");
lastname[3] = ("Smithe-Jonese");
lastname[4] = ("Bigglesworth");
lastname[5] = ("Moo");
lastname[6] = ("Diesel");
lastname[7] = ("Mangosteen");


local fname;
local lname;

local setn = false;
while (setn == false) {
if (AICompany.GetPresidentGender(Me) == 0) {
fname = firstnamem[AIBase.RandRange(8)];
} else {
fname = firstnamef[AIBase.RandRange(8)];
}
lname = lastname[AIBase.RandRange(8)];

if (AICompany.SetPresidentName(fname + " " + lname)) {
setn = true;
}
}

local companynames = [0,1,2,3,4,5,6,7]

companynames[0] = (HomeTownName + " Garages");
companynames[1] = ("Transport for " + HomeTownName);
companynames[2] = (HomeTownName + " Engineering");
companynames[3] = (HomeTownName + " Transport");
companynames[4] = ("Buses by " + fname);
companynames[5] = (fname + "'s Charabanc");
companynames[6] = (HomeTownName + " Express");
companynames[7] = (lname + " & Co.");


local set = false;
while (set == false) {
if (AICompany.SetName(companynames[AIBase.RandRange(8)]))
{
set = true;
}
}



return
}

// ====================================================== 
//                     BUILD HQ
// ====================================================== 

function CivilAI::BuildHQ(hometown)
{
// Let's (in)elegantly brute-force the HQ!

  local HomeTownGrid = [0,0];
  local HQ = 0;

local towntile = AITown.GetLocation(hometown);
HomeTownGrid[0] = AIMap.GetTileX(towntile);
HomeTownGrid[1] = AIMap.GetTileY(towntile);

local trytilegrid = [0,0]
local trytile = AIMap.GetTileIndex(trytilegrid[0],trytilegrid[1]);

// this is a cross search!

local i = 0; // iteration
local s = 3; // initial cross length
local j = AIBase.RandRange(2); // toggle x/y
local k = AIBase.RandRange(4); // toggle +/-

while (!AICompany.BuildCompanyHQ(trytile)) {

if (j == 0) {
	trytilegrid[1] = HomeTownGrid[1] + 2;
	if (k < 2) {
	trytilegrid[0] = (HomeTownGrid[0] + s);	
	} else {
    trytilegrid[0] = (HomeTownGrid[0] - s);
	}
} else {
	trytilegrid[0] = HomeTownGrid[0] + 2;
	if (k < 2) {
	trytilegrid[1] = (HomeTownGrid[1] + s);	
	} else {
    trytilegrid[1] = (HomeTownGrid[1] - s);	
	}
}

trytile = AIMap.GetTileIndex(trytilegrid[0],trytilegrid[1]);
// AISign.BuildSign(trytile, "I tried to build an HQ here");

if (j < 1) {
		j = j + 1;
		} else {
		j = 0;
		}
if (k < 3) {
		k = k + 1;
		} else {
		k = 0;
		}
if (i < 3) {
		i = i + 1;
		} else {
		i = 0;
		s = s + 5;
		}		

}
HQ = trytile;
AILog.Info("I've built an HQ at " + trytilegrid[0] + " ," + trytilegrid[1] + ". We're off to a good start!");	


BuildHomeDepot(HQ,hometown);
BuildBuses(hometown,HomeDepot);
return
}

// ====================================================== 
//                 PLACE HOME DEPOT
// ====================================================== 

function CivilAI::BuildHomeDepot(hq,hometown)
{
AILog.Info("I'll build my first depot as close to the HQ as possible.")

CashUp(); // take out maximum loan to start

// clockwise spiral out
local trytilegrid = [AIMap.GetTileX(hq),AIMap.GetTileY(hq)]
local trytile;
local x = 1
local y = 1
local i = 0
while ((HomeDepot == 0) && (i < 20)) {
//y--
for (;y >= 0-i;y--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, "?");
//AILog.Info("testing tile at " + x + "," + y);
if (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD) 
&& !AIRoad.IsRoadDepotTile(trytile)
&& !AIRoad.IsRoadStationTile(trytile)
&& !AIRoad.IsDriveThroughRoadStationTile(trytile)
&& !AIBridge.IsBridgeTile(trytile)
&& !HomeDepot)
HomeDepot = BuildDepot(trytile,hometown);
}
//x--;
for (;x >= 0-i;x--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, "?");
//AILog.Info("testing tile at " + x + "," + y);
if (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD) 
&& !AIRoad.IsRoadDepotTile(trytile)
&& !AIRoad.IsRoadStationTile(trytile)
&& !AIRoad.IsDriveThroughRoadStationTile(trytile)
&& !AIBridge.IsBridgeTile(trytile)
&& !HomeDepot)
HomeDepot = BuildDepot(trytile,hometown);
}
//y++;
for (;y <= i;y++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, "?");
//AILog.Info("testing tile at " + x + "," + y);
if (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD)
&& !AIRoad.IsRoadDepotTile(trytile)
&& !AIRoad.IsRoadStationTile(trytile)
&& !AIRoad.IsDriveThroughRoadStationTile(trytile)
&& !AIBridge.IsBridgeTile(trytile)
&& !HomeDepot)
HomeDepot = BuildDepot(trytile,hometown);
}
//x++;
for (;x <= i;x++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, "?");
//AILog.Info("testing tile at " + x + "," + y);
if (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD)
&& !AIRoad.IsRoadDepotTile(trytile)
&& !AIRoad.IsRoadStationTile(trytile)
&& !AIRoad.IsDriveThroughRoadStationTile(trytile)
&& !AIBridge.IsBridgeTile(trytile)
&& !HomeDepot)
HomeDepot = BuildDepot(trytile,hometown);
}
i=i+1
}

if (HomeDepot) {
AILog.Info("I've built my home depot at " + AIMap.GetTileX(HomeDepot) + "," + AIMap.GetTileY(HomeDepot) + "."); 
} else {
AILog.Info("I've failed to build a depot."); 
Failure();
}
return
}

// ====================================================== 
//             FIND HOME DEPOT (after load)
// ====================================================== 


function CivilAI::FindHomeDepot()
{
local hq = AICompany.GetCompanyHQ(Me);
if (hq != -1) {

AILog.Info("I'm looking for my home depot. It should be near my headquarters in " + AITown.GetName(AITile.GetClosestTown(hq)) + ".")

// clockwise spiral out
local trytilegrid = [AIMap.GetTileX(hq),AIMap.GetTileY(hq)]
local trytile;
local x = 1
local y = 1
local i = 0
while ((HomeDepot == 0) && (i < 20)) {
//y--
for (;y >= 0-i;y--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);

if (AIRoad.IsRoadDepotTile(trytile)
	&& AICompany.IsMine(AITile.GetOwner(trytile))
	&& !HomeDepot)
	{
	HomeDepot = trytile;
	}
}
//x--;
for (;x >= 0-i;x--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);


if (AIRoad.IsRoadDepotTile(trytile) 
	&& AICompany.IsMine(AITile.GetOwner(trytile))
	&& !HomeDepot)
	{
	HomeDepot = trytile;
	}
}
//y++;
for (;y <= i;y++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);

if (AIRoad.IsRoadDepotTile(trytile)
	&& AICompany.IsMine(AITile.GetOwner(trytile))
	&& !HomeDepot)
	{
	HomeDepot = trytile;
	}
}
//x++;
for (;x <= i;x++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);

if (AIRoad.IsRoadDepotTile(trytile)  
	&& AICompany.IsMine(AITile.GetOwner(trytile))
	&& !HomeDepot)
	{
	HomeDepot = trytile;
	}
}
i=i+1
}

if (HomeDepot) {
AILog.Info("I've found my home depot at " + AIMap.GetTileX(HomeDepot) + "," + AIMap.GetTileY(HomeDepot) + ".") 

if (AICompany.GetName(Me) == "Failed CivilAI") { SetName(AITown.GetName(AITile.GetClosestTown(hq))) } // resurrect formerly failed AI.

} else {
AILog.Info("I can't find my home depot."); 
Failure();
}
} else {
AILog.Info("I can't find my headquarters."); 
Failure();
}
return
}
// ====================================================== 
//            CHECK HOME DEPOT IS CONNECTED
// ====================================================== 
function CivilAI::HomeConnect() {

AILog.Info("I'm checking my home depot is still connected to the road network.");


// check all our depots are connected to the tile in front of them
local dlist = AIDepotList(AITile.TRANSPORT_ROAD); 	
foreach (depot, z in dlist) {
AIRoad.BuildRoad (AIRoad.GetRoadDepotFrontTile(depot),depot);
//AILog.Info("Checking immediate connection at depot " + depot)
}


local hq = AICompany.GetCompanyHQ(Me);
local a = [AIRoad.GetRoadDepotFrontTile(HomeDepot)]
local b = [AITown.GetLocation(AITile.GetClosestTown(hq))]

local buildroad = RoadPF();
buildroad.cost.max_cost 			= 10000000;   //10000000;
buildroad.cost.tile     			= 100;        // 100;
buildroad.cost.no_existing_road		= 75;         //40;
buildroad.cost.turn           		= 100;        //100;
buildroad.cost.slope          		= 400;        //200;
buildroad.cost.bridge_per_tile		= 500;        //150;
buildroad.cost.tunnel_per_tile		= 900;        //120;
buildroad.cost.coast       			= 100;        //20;
buildroad.cost.max_bridge_length	= 10;          //10;
buildroad.cost.max_tunnel_length	= 20;          //20;

buildroad.InitializePath(a, b);

local path = false;
while (path == false) {
  path = buildroad.FindPath(50);
  AIController.Sleep(1);
}


if (path == null) {
AILog.Info("It isn't.")
Failure();
return
}
while (path != null) {
  local par = path.GetParent();
  if (par != null) {
    local last_node = path.GetTile();
    if (AIMap.DistanceManhattan(path.GetTile(), par.GetTile()) == 1 ) {
      if (!AIRoad.BuildRoad(path.GetTile(), par.GetTile())) {
        /* An error occurred while building a piece of road. TODO: handle it. 
         * Note that this could mean the road was already built. */
      }
    } else {
      /* Build a bridge or tunnel. */
      if (!AIBridge.IsBridgeTile(path.GetTile()) && !AITunnel.IsTunnelTile(path.GetTile())) {
        /* If it was a road tile, demolish it first. Do this to work around expended roadbits. */
        if (AIRoad.IsRoadTile(path.GetTile())) AITile.DemolishTile(path.GetTile());
        if (AITunnel.GetOtherTunnelEnd(path.GetTile()) == par.GetTile()) {
          if (!AITunnel.BuildTunnel(AIVehicle.VT_ROAD, path.GetTile())) {
            /* An error occured while building a tunnel. TODO: handle it. */
          }
        } else {
          local bridge_list = AIBridgeList_Length(AIMap.DistanceManhattan(path.GetTile(), par.GetTile()) + 1);
          bridge_list.Valuate(AIBridge.GetMaxSpeed);
          bridge_list.Sort(AIList.SORT_BY_VALUE, false);
          if (!AIBridge.BuildBridge(AIVehicle.VT_ROAD, bridge_list.Begin(), path.GetTile(), par.GetTile())) {
            /* An error occured while building a bridge. TODO: handle it. */
          }
        }
      }
    }
  }
  path = par;
}
AILog.Info("It's fine.")

}

// ====================================================== 
//                    BUILD DEPOT
// ====================================================== 



function CivilAI::BuildDepot(roadtile,town) {
// this is road, check it's joined to the centre of town
local testroad = RoadPF();
local towntile = AITown.GetLocation(town);
local depot = 0;

testroad.cost.no_existing_road = testroad.cost.max_cost; // only check existing roads
testroad.InitializePath([roadtile], [towntile]);

local path = false;
while (path == false) {
path = testroad.FindPath(20);
AIController.Sleep(1);
}

if ((path != null) && (AITile.GetSlope(roadtile) == 0)) { 
// AISign.BuildSign(roadtile, "?");

local builtlink = false; // keep trying if a vehicle is preventing building the road connection
local c = 0;
local roadtilegrid = [AIMap.GetTileX(roadtile),AIMap.GetTileY(roadtile)]
    if (AIRoad.BuildRoadDepot (AIMap.GetTileIndex(roadtilegrid[0]+1,roadtilegrid[1]),roadtile)) {
	depot = AIMap.GetTileIndex(roadtilegrid[0]+1,roadtilegrid[1]);
    while (!builtlink && c < 100) {
	builtlink = AIRoad.BuildRoad (roadtile,depot);
	c++
	AIController.Sleep(5);
	}	
	} else if (AIRoad.BuildRoadDepot (AIMap.GetTileIndex(roadtilegrid[0]-1,roadtilegrid[1]),roadtile)) {
	depot = AIMap.GetTileIndex(roadtilegrid[0]-1,roadtilegrid[1]);
    while (!builtlink && c < 100) {
	builtlink = AIRoad.BuildRoad (roadtile,depot);
	c++
	AIController.Sleep(5);
	}	
	} else if (AIRoad.BuildRoadDepot (AIMap.GetTileIndex(roadtilegrid[0],roadtilegrid[1]+1),roadtile)) {
	depot = AIMap.GetTileIndex(roadtilegrid[0],roadtilegrid[1]+1);	 
    while (!builtlink && c < 100) {
	builtlink = AIRoad.BuildRoad (roadtile,depot);
	c++
	AIController.Sleep(5);
	}	
	} else if (AIRoad.BuildRoadDepot (AIMap.GetTileIndex(roadtilegrid[0],roadtilegrid[1]-1),roadtile)) {
	depot = AIMap.GetTileIndex(roadtilegrid[0],roadtilegrid[1]-1);
    while (!builtlink && c < 100) {
	builtlink = AIRoad.BuildRoad (roadtile,depot);
	c++
	AIController.Sleep(5);
	}	
	}	
}
return depot;
}


//<
// ====================================================== 
// ====================================================== 
//      		   BBBB     U    U    SSSS
//     			   B   BB   U    U   SS   S
//      		   BBBBB    U    U     SS
//       		   B   BB   U    U   S   SS
//       		   BBBB      UUUU     SSSS
// ====================================================== 
// ====================================================== 
//>
// ====================================================== 
//                     BUILD BUS ROUTE
// ====================================================== 

function CivilAI::BuildBuses(town,depot) {
AILog.Info("I'm going to provide local bus services in " + AITown.GetName(town) + "."); 

local HasBus;
local statc;
local statn;
local state;
local statw;
local stats;

local statcount = 0;
local route1;
local route2;

// check if we have buses

HasBus = IdentifyBus(false);
if (!HasBus) {
return
}

// build central bus stop

statc = BuildBusStop (0,0,3,3,town,false,false);

if (!statc) { 
AILog.Info("I've failed to build a central bus stop in " + AITown.GetName(town) + ". Better luck next year, perhaps."); 
if (depot != HomeDepot) (AIRoad.RemoveRoadDepot(depot));

return
} else {
AILog.Info("I've built a central bus stop: " + AIBaseStation.GetName(statc-1) + "."); 
}


// build outer bus stops

statn = BuildBusStop (-10,0,1,5,town,false,true);

if (!statn) { 
//AILog.Info("I've failed to build a northern bus stop in " + AITown.GetName(town) + "."); 
} else {
AILog.Info("I've built a northern bus stop: " + AIBaseStation.GetName(statn-1) + "."); 
statcount++
}


state = BuildBusStop (0,10,1,5,town,false,true);

if (!state) { 
//AILog.Info("I've failed to build an eastern bus stop in " + AITown.GetName(town) + "."); 
} else {
AILog.Info("I've built an eastern bus stop: " + AIBaseStation.GetName(state-1) + "."); 
statcount++
}

stats = BuildBusStop (10,0,1,5,town,false,true);

if (!stats) { 
//AILog.Info("I've failed to build a southern bus stop in " + AITown.GetName(town) + "."); 
} else {
AILog.Info("I've built a southern bus stop: " + AIBaseStation.GetName(stats-1) + "."); 
statcount++
}

statw = BuildBusStop (0,-10,1,5,town,false,true);

if (!statw) { 
//AILog.Info("I've failed to build a western bus stop in " + AITown.GetName(town) + "."); 
} else {
AILog.Info("I've built a western bus stop: " + AIBaseStation.GetName(statw-1) + "."); 
statcount++
}

// okay, let's see what we've got here; time to construct some routes!

if (statcount == 4) {

AILog.Info("I've built five bus stops in " + AITown.GetName(town) + " - great!"); 

CreateRoute(HasBus,1,[statn,statc,stats,statc],depot,(AITown.GetName(town)),false,0,0);
CreateRoute(HasBus,1,[statw,statc,state,statc],depot,(AITown.GetName(town)),false,0,0);

} else if (statcount == 3) {

AILog.Info("Four stops, that's nice."); 

//identify the missing stop and do routes
if (!statn) {
CreateRoute(HasBus,2,[state,statc,statw,statc,stats,statc],depot,(AITown.GetName(town)),false,0,0);
} else if (!state) {
CreateRoute(HasBus,2,[statn,statc,statw,statc,stats,statc],depot,(AITown.GetName(town)),false,0,0);
} else if (!statw) {
CreateRoute(HasBus,2,[statn,statc,state,statc,stats,statc],depot,(AITown.GetName(town)),false,0,0);
} else {
CreateRoute(HasBus,2,[state,statc,statw,statc,statn,statc],depot,(AITown.GetName(town)),false,0,0);
}

} else if (statcount == 2) {

AILog.Info("I can make a route out of these three stops."); 

if (!statn && !state) {
CreateRoute(HasBus,1,[statw,statc,stats,statc],depot,(AITown.GetName(town)),false,0,0);
} else if (!statn && !statw) {
CreateRoute(HasBus,1,[stats,statc,state,statc],depot,(AITown.GetName(town)),false,0,0);
} else if (!statn && !stats) {
CreateRoute(HasBus,1,[statw,statc,state,statc],depot,(AITown.GetName(town)),false,0,0);
} else if (!state && !statw) {
CreateRoute(HasBus,1,[statn,statc,stats,statc],depot,(AITown.GetName(town)),false,0,0);
} else if (!state && !stats) {
CreateRoute(HasBus,1,[statw,statc,statn,statc],depot,(AITown.GetName(town)),false,0,0);
} else if (!statw && !stats) {
CreateRoute(HasBus,1,[state,statc,statn,statc],depot,(AITown.GetName(town)),false,0,0);

}

} else {
AILog.Info("I couldn't build enough bus stops in " + AITown.GetName(town) + ". Better luck next year, perhaps.");
//if (depot != HomeDepot) (AIRoad.RemoveRoadDepot(depot)); // don't remove depots now, we're keeping them for towncars.
}
}

// ====================================================== 
//                  BUY INTERCITY BUS
// ====================================================== 

function CivilAI::InterCity() {

local dosh = AICompany.GetBankBalance(Me);
local nbus = IdentifyCoach(true);

if (nbus == false) {
return;
}
else if (dosh > (AIEngine.GetPrice(nbus-1) * 2)) {

// check we don't have MaxBus
local rvlist = AIVehicleList();
foreach (veh,z in rvlist) {
if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_ROAD) {
rvlist.RemoveItem(veh);
}
}
rvlist.Valuate(AIVehicle.GetCapacity, 0);
rvlist.KeepAboveValue(9);
local buscount = rvlist.Count();
if (buscount < MaxBus) {

local HasBus = IdentifyCoach(false);
if (!HasBus) {
return
}
AILog.Info("I'm looking for an intercity bus route.");

local bslist = AIStationList(AIStation.STATION_BUS_STOP); 
bslist.Valuate(AIStation.GetCargoWaiting, 0); //order by pax waiting
bslist.RemoveBelowValue(100);
bslist.Valuate(AIBase.RandItem); // shuffle the list

foreach (stop,z in bslist) {
local vlist = AIVehicleList_Station(stop);
foreach (veh,z in vlist) {
if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_ROAD) {
vlist.RemoveItem(veh);
}
}
if (vlist.Count() > 30) {					// remove saturated stops
//AILog.Info(AIBaseStation.GetName(stop) + " has too many buses already.");
bslist.RemoveItem(stop);
}
}
if (bslist.Count() < 2) {
AILog.Info("I couldn't find one.");
return
}
local stat1 = bslist.Begin();
local stat2 = null;
bslist.RemoveTop(1);
foreach (stop,z in bslist) {
if ((AIStation.GetNearestTown(stop)) != (AIStation.GetNearestTown(stat1))) {
stat2 = stop
break;
}
}
if (stat2 == null) {
AILog.Info("I couldn't find one.");
return
}
local paxo = (((AIStation.GetCargoWaiting(stat1,0) + AIStation.GetCargoWaiting(stat2,0)) / 1000 ) + 1)


if (paxo == 1){ 
AILog.Info("I'm buying " + paxo + " bus to run from " + (AIBaseStation.GetName(stat1)) + " to " + (AIBaseStation.GetName(stat2)) + ".");
} else {
AILog.Info("I'm buying " + paxo + " buses to run from " + (AIBaseStation.GetName(stat1)) + " to " + (AIBaseStation.GetName(stat2)) + ".");
}

CreateRoute(HasBus,paxo,[stat1+1,stat2+1],HomeDepot,(AITown.GetName(AIStation.GetNearestTown(stat1))),false,0,0);

} else {
AILog.Info("I've already got my maximum number of buses, so I won't build more.")
}
} else {
AILog.Info("I don't have the money for new buses at the moment.")
}


return
}


// ====================================================== 
//                BUILD/REMOVE BUS STOP
// ====================================================== 

function CivilAI::BuildBusStop(xoff,yoff,count,spread,town,rem,spike) {

// clockwise spiral out
local trytilegrid = [((AIMap.GetTileX(AITown.GetLocation(town)))+xoff),((AIMap.GetTileY(AITown.GetLocation(town)))+yoff)]
local trytile;
local aimx; // aimtile
local aimy; // aimtile
local aimxm; // an additional aimtile, for bridge approach tests (we don't build on bridge approaches, because they send us on journeys)
local aimym; // an additional aimtile, for bridge approach tests (we don't build on bridge approaches, because they send us on journeys)
local testroad = RoadPF();
testroad.cost.no_existing_road = testroad.cost.max_cost;
local x = 0
local y = 0
local i = 0
local stat = AIStation.STATION_NEW

local statcount = 0

while ((i < spread) && (statcount < count)) {
//y--
for (;y >= 0-i;y--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
aimx  = AIMap.GetTileIndex(trytilegrid[0]+x+1,trytilegrid[1]+y);
aimy  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y+1);
aimxm  = AIMap.GetTileIndex(trytilegrid[0]+x-1,trytilegrid[1]+y);
aimym  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y-1);

testroad.InitializePath([trytile], [AITown.GetLocation(town)]);
local path = false;
while (path == false) {
path = testroad.FindPath(20);
AIController.Sleep(1);
}

if (rem) {
AIRoad.RemoveRoadStation(trytile) } else
if ((path != null) && (statcount < count) && (AITile.GetCargoAcceptance(trytile, 0, 1, 1, 3) > 11) && (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD))) {
	if (!AIBridge.IsBridgeTile(aimx) && !AIBridge.IsBridgeTile(aimxm) && 
	(AITile.GetSlope(aimx) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimxm) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimx,AIRoad.ROADVEHTYPE_BUS,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		if (spike) { 
		SpikeX(trytile,town); 
		} 
		}
	else if (!AIBridge.IsBridgeTile(aimy) && !AIBridge.IsBridgeTile(aimym) && 
	(AITile.GetSlope(aimy) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimym) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimy,AIRoad.ROADVEHTYPE_BUS,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		if (spike) { 
		SpikeY(trytile,town); 
		} 
		}
}
}

//x--;
for (;x >= 0-i;x--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
aimx  = AIMap.GetTileIndex(trytilegrid[0]+x+1,trytilegrid[1]+y);
aimy  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y+1);
aimxm  = AIMap.GetTileIndex(trytilegrid[0]+x-1,trytilegrid[1]+y);
aimym  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y-1);

testroad.InitializePath([trytile], [AITown.GetLocation(town)]);
local path = false;
while (path == false) {
path = testroad.FindPath(20);
AIController.Sleep(1);
}
if (rem) {
AIRoad.RemoveRoadStation(trytile) } else
if ((path != null) && (statcount < count) && (AITile.GetCargoAcceptance(trytile, 0, 1, 1, 3) > 11) && (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD))) {
	if (!AIBridge.IsBridgeTile(aimx) && !AIBridge.IsBridgeTile(aimxm) && 
	(AITile.GetSlope(aimx) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimxm) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimx,AIRoad.ROADVEHTYPE_BUS,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		if (spike) { 
		SpikeX(trytile,town); 
		} 
		}
	else if (!AIBridge.IsBridgeTile(aimy) && !AIBridge.IsBridgeTile(aimym) && 
	(AITile.GetSlope(aimy) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimym) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimy,AIRoad.ROADVEHTYPE_BUS,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		if (spike) { 
		SpikeY(trytile,town); 
		} 
		}
}
}

//y++;
for (;y <= i;y++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
aimx  = AIMap.GetTileIndex(trytilegrid[0]+x+1,trytilegrid[1]+y);
aimy  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y+1);
aimxm  = AIMap.GetTileIndex(trytilegrid[0]+x-1,trytilegrid[1]+y);
aimym  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y-1);

testroad.InitializePath([trytile], [AITown.GetLocation(town)]);
local path = false;
while (path == false) {
path = testroad.FindPath(20);
AIController.Sleep(1);
}
if (rem) {
AIRoad.RemoveRoadStation(trytile) } else
if ((path != null) && (statcount < count) && (AITile.GetCargoAcceptance(trytile, 0, 1, 1, 3) > 11) && (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD))) {
	if (!AIBridge.IsBridgeTile(aimx) && !AIBridge.IsBridgeTile(aimxm) && 
	(AITile.GetSlope(aimx) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimxm) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimx,AIRoad.ROADVEHTYPE_BUS,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		if (spike) { 
		SpikeX(trytile,town); 
		} 
		}
	else if (!AIBridge.IsBridgeTile(aimy) && !AIBridge.IsBridgeTile(aimym) && 
	(AITile.GetSlope(aimy) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimym) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimy,AIRoad.ROADVEHTYPE_BUS,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		if (spike) { 
		SpikeY(trytile,town); 
		} 
		}
}
}

//x++;
for (;x <= i+1;x++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
aimx  = AIMap.GetTileIndex(trytilegrid[0]+x+1,trytilegrid[1]+y);
aimy  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y+1);
aimxm  = AIMap.GetTileIndex(trytilegrid[0]+x-1,trytilegrid[1]+y);
aimym  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y-1);

testroad.InitializePath([trytile], [AITown.GetLocation(town)]);
local path = false;
while (path == false) {
path = testroad.FindPath(20);
AIController.Sleep(1);
}
if (rem) {
AIRoad.RemoveRoadStation(trytile) } else
if ((path != null) && (statcount < count) && (AITile.GetCargoAcceptance(trytile, 0, 1, 1, 3) > 11) && (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD))) {
	if (!AIBridge.IsBridgeTile(aimx) && !AIBridge.IsBridgeTile(aimxm) && 
	(AITile.GetSlope(aimx) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimxm) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimx,AIRoad.ROADVEHTYPE_BUS,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		if (spike) { 
		SpikeX(trytile,town); 
		} 
		}
	else if (!AIBridge.IsBridgeTile(aimy) && !AIBridge.IsBridgeTile(aimym) && 
	(AITile.GetSlope(aimy) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimym) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimy,AIRoad.ROADVEHTYPE_BUS,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		if (spike) { 
		SpikeY(trytile,town); 
		} 
		}
}
}
i=i+1
}
if (statcount > 0) {

// try and put a truckstop for water, if required
local c = FindCargo("WATR");
if (c > 0) {
local towerlist = AIIndustryList_CargoAccepting (c);
foreach (tower, z in towerlist) {
if (AITileList_IndustryAccepting(tower, 3).HasItem(AIBaseStation.GetLocation(stat))) {
BuildTruckStop(stat,AIBaseStation.GetLocation(stat))
}
}
}


return stat+1 } else { return false }
}

// ====================================================== 
//                BUILD TURNAROUND SPIKES
// ====================================================== 

function CivilAI::SpikeX(tile,town) {

local x = AIMap.GetTileX(tile)
local y = AIMap.GetTileY(tile)
local townx = AIMap.GetTileX(AITown.GetLocation(town))

if (x >= townx) {
AIRoad.BuildRoadFull(
AIMap.GetTileIndex(x+1,y+1),
AIMap.GetTileIndex(x+1,y)
); 
AIRoad.BuildRoadFull(
AIMap.GetTileIndex(x+1,y-1),
AIMap.GetTileIndex(x+1,y)
);
AIRoad.BuildRoadFull(
AIMap.GetTileIndex(x+2,y),
tile
);
} else {
AIRoad.BuildRoadFull(
AIMap.GetTileIndex(x-1,y+1),
AIMap.GetTileIndex(x-1,y)
); 
AIRoad.BuildRoadFull(
AIMap.GetTileIndex(x-1,y-1),
AIMap.GetTileIndex(x-1,y)
);
AIRoad.BuildRoadFull(
AIMap.GetTileIndex(x-2,y),
tile
);
}
return
}

function CivilAI::SpikeY(tile,town) {

local x = AIMap.GetTileX(tile)
local y = AIMap.GetTileY(tile)
local towny = AIMap.GetTileY(AITown.GetLocation(town))

if (y >= towny) {
AIRoad.BuildRoadFull(
AIMap.GetTileIndex(x+1,y+1),
AIMap.GetTileIndex(x,y+1)
);
AIRoad.BuildRoadFull(
AIMap.GetTileIndex(x-1,y+1),
AIMap.GetTileIndex(x,y+1)
);
AIRoad.BuildRoadFull(
AIMap.GetTileIndex(x,y+2),
tile
);
} else {
AIRoad.BuildRoadFull(
AIMap.GetTileIndex(x+1,y-1),
AIMap.GetTileIndex(x,y-1)
);
AIRoad.BuildRoadFull(
AIMap.GetTileIndex(x-1,y-1),
AIMap.GetTileIndex(x,y-1)
);
AIRoad.BuildRoadFull(
AIMap.GetTileIndex(x,y-2),
tile
);
}
return
}



// ====================================================== 
//                    FIND BUSES TO BUY
// ======================================================  


function CivilAI::IdentifyBus(silent) {
local busid= 0;
local vlist= 0;
local blist= 0;
local bcap = 0;
local bdate= 0;
local z = 0;

vlist = AIEngineList(AIVehicle.VT_ROAD); 	// list of all road vehicles

foreach (rv,z in vlist) {

//AILog.Info("Assessing " + AIEngine.GetName(rv));

if ((AIEngine.IsBuildable(rv)) 
&& (AIEngine.CanRefitCargo(rv, 0))					// again with the assumption that pax = 0
&& (AIEngine.GetCapacity(rv) > 9)	
&& (AIEngine.GetRoadType(rv) == AIRoad.ROADTYPE_ROAD)) {

if (AIEngine.GetCapacity(rv) > bcap) {				    // highest capacity.
// AILog.Info("!");
 busid = rv+1	
 bcap = AIEngine.GetCapacity(rv)
 bdate =  AIEngine.GetDesignDate(rv)
  } else if ((AIEngine.GetCapacity(rv) == bcap) &&  
		   (AIEngine.GetDesignDate(rv) > bdate))	{	// If a tie, we want the newest.
 busid = rv+1	
 bcap = AIEngine.GetCapacity(rv)
 bdate =  AIEngine.GetDesignDate(rv)
  }
  }
  }
  
if (busid > 0) {
if (!silent) AILog.Info("I've selected " + AIEngine.GetName(busid-1)  + " for local buses.")
return (busid);
} else {
if (!silent) AILog.Info("I can't find any buses available to buy...")
return false;
}
}

// ====================================================== 
//                    FIND COACHES TO BUY
// ====================================================== 

function CivilAI::IdentifyCoach(silent) {
local busid= 0;
local vlist= 0;
local blist= 0;
local bcap = 0;
local bdate= 0;
local z = 0;

vlist = AIEngineList(AIVehicle.VT_ROAD); 	// list of all road vehicles

foreach (rv,z in vlist) {

//AILog.Info("Assessing " + AIEngine.GetName(rv));

if ((AIEngine.IsBuildable(rv)) 
&& (AIEngine.CanRefitCargo(rv, 0))					// again with the assumption that pax = 0
&& (AIEngine.GetCapacity(rv) > 9)	
&& (AIEngine.GetRoadType(rv) == AIRoad.ROADTYPE_ROAD)) {

if (AIEngine.GetMaxSpeed(rv) > bcap) {				    // highest speed.
// AILog.Info("!");
 busid = rv+1	
 bcap = AIEngine.GetMaxSpeed(rv)
 bdate =  AIEngine.GetDesignDate(rv)
  } else if ((AIEngine.GetCapacity(rv) == bcap) &&  
		   (AIEngine.GetDesignDate(rv) > bdate))	{	// If a tie, we want the newest.
 busid = rv+1	
 bcap = AIEngine.GetMaxSpeed(rv)
 bdate =  AIEngine.GetDesignDate(rv)
  }
  }
  }
if (busid > 0) {
if (!silent) AILog.Info("I've selected " + AIEngine.GetName(busid-1)  + " for intercity buses.")
return (busid);
} else {
if (!silent) AILog.Info("I can't find any buses available to buy...")
return false;
}
}  

// ====================================================== 
//                  BUY BUSES FOR ROUTE
// ====================================================== 

function CivilAI::CreateRoute(vehicle,count,stops,depot,routename,fullload,cargo,group) {

local obus
local clonebus
local stop
local nx
local name
local changename = false
local cload = true;


obus = (AIVehicle.BuildVehicle(depot,vehicle-1)); 
if (AIVehicle.IsValidVehicle(obus)) {
foreach (stop in stops) {
if (fullload) {
if (cload) {
AIOrder.AppendOrder(obus, AIBaseStation.GetLocation(stop-1),(AIOrder.OF_NON_STOP_INTERMEDIATE + AIOrder.OF_FULL_LOAD_ANY))
cload = false; // only full load at first station
} else {
AIOrder.AppendOrder(obus, AIBaseStation.GetLocation(stop-1),(AIOrder.OF_NON_STOP_INTERMEDIATE + AIOrder.OF_NO_LOAD))
}
} else {
AIOrder.AppendOrder(obus, AIBaseStation.GetLocation(stop-1),(AIOrder.OF_NON_STOP_INTERMEDIATE))
}
}
nx = 0
name = null
changename = false
while (!changename && (nx < 1000)) {
nx++ 
name = (routename + " " + nx)
changename = AIVehicle.SetName(obus,name)
}
AILog.Info("I've bought a vehicle for " + routename + ".")
AIGroup.MoveVehicle(Groups[group], obus); // add to appropriate group
AIVehicle.RefitVehicle(obus, cargo); // refit to correct cargo
AIVehicle.StartStopVehicle(obus);

} else {
AILog.Info("For some reason (probably not enough money), I couldn't buy a vehicle for " + routename + ".")
}

for (local c = 1; c < count; c++) {
if (AIVehicle.IsValidVehicle(obus)) { // extra if check - don't try cloning if we failed the first time, produces buses with no orders.
clonebus = AIVehicle.BuildVehicle(depot,vehicle-1)
if (AIVehicle.IsValidVehicle(clonebus)) {
AIOrder.ShareOrders(clonebus,obus);

nx = 0
name = null
changename = false
while (!changename && (nx < 1000)) {
nx++ 
name = (routename + " " + nx)
changename = AIVehicle.SetName(clonebus,name)
}
AIOrder.SkipToOrder(clonebus,AIBase.RandRange(AIOrder.GetOrderCount(clonebus)));
AILog.Info("I've bought another vehicle for " + routename + ".")
AIGroup.MoveVehicle(Groups[group], clonebus); // add to appropriate group
AIVehicle.RefitVehicle(clonebus, cargo); // refit to correct cargo
AIVehicle.StartStopVehicle(clonebus);
}
} else {
AILog.Info("For some reason (probably not enough money), I couldn't buy a vehicle for " + routename + ".")

}
}

return
}

// ====================================================== 
//                 REVIEW BUS SERVICES
// ====================================================== 

function CivilAI::BusReview() {

AILog.Info("I'm reviewing my bus services.");	


// remove unused stops
local bslist = AIStationList(AIStation.STATION_BUS_STOP); 
local vlist
foreach (stop,z in bslist) {
vlist = AIVehicleList_Station(stop)
foreach (veh,z in vlist) {
if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_ROAD) {
vlist.RemoveItem(veh);
}
} 
if (vlist.Count() == 0) {	
AILog.Info((AIBaseStation.GetName(stop)) + " has no service. I'll remove the stop.")
if (!AIRoad.RemoveRoadStation(AIBaseStation.GetLocation(stop))) {
BuildBusStop (0,0,4,3,(AIStation.GetNearestTown(stop)),true,false); // remove central stops.
}
}
}

local dosh = AICompany.GetBankBalance(Me);
local nbus = IdentifyBus(true);

if (nbus == false) {

return;
}
else if (dosh > (AIEngine.GetPrice(nbus-1) * 2)) {


// replacing old buses?
local rvlist
rvlist = AIVehicleList();

for (local v = rvlist.Begin(); !(rvlist.IsEnd()); v = rvlist.Next()) {
if  ((AIVehicle.GetCapacity(v,0) > 9) &&
	((AIVehicle.GetAgeLeft(v)) < (365 * 1))) {

// clone the old bus

local obus = v;
local upbus

// compare orders; if first two orders are in different towns, build a coach instead of a bus
local order1 = AIOrder.GetOrderDestination(obus, 0)
local order2 = AIOrder.GetOrderDestination(obus, 1)
if (AITile.GetClosestTown(order1) == AITile.GetClosestTown(order2)) {
upbus = IdentifyBus(true);
} else {
upbus = IdentifyCoach(true);
}

local clonebus = AIVehicle.BuildVehicle(HomeDepot,upbus-1)
if (AIVehicle.IsValidVehicle(clonebus)) {
//AILog.Info("I've bought a replacement for " + (AIVehicle.GetName(v)) + ".")
AIOrder.ShareOrders(clonebus,obus);
local nx = 0
local name = null
local changename = false

local stopp = (AIStation.GetStationID(AIOrder.GetOrderDestination(obus,0)));

while (!changename && (nx < 1000)) {
nx++ 
name = ((AITown.GetName(AIStation.GetNearestTown(stopp))) + " " + nx)
changename = AIVehicle.SetName(clonebus,name)
}
AIOrder.SkipToOrder(clonebus,AIBase.RandRange(6));
AIGroup.MoveVehicle(Groups[0], clonebus); // add to appropriate group
AIVehicle.StartStopVehicle(clonebus);

AIVehicle.SendVehicleToDepot(obus);
AILog.Info(AIVehicle.GetName(obus) + " is getting old, so I'm replacing it.")

} else {
AILog.Info("For some reason (probably not enough money), I couldn't buy a replacement for " + (AIVehicle.GetName(obus)) + ".")
}
}
}



// do we want to add more buses?

// check we don't have MaxBus
local rvlist = AIVehicleList();
foreach (veh,z in rvlist) {
if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_ROAD) {
rvlist.RemoveItem(veh);
}
}
rvlist.Valuate(AIVehicle.GetCapacity, 0);
rvlist.KeepAboveValue(9);
local buscount = rvlist.Count();
if (buscount < MaxBus) {


local vlist
local clonebus
local obus
local upbus
local bslist = AIStationList(AIStation.STATION_BUS_STOP); 

// bus review
foreach (stop,z in bslist) {
if ((AIStation.GetCargoWaiting(stop,0) > 500) 
   && AIStation.GetCargoRating(stop,0) < 60
    ) 
	{	
// find a bus that uses this stop and copy it in the home depot	
//AILog.Info("Assessing " + AIStation.GetName(stop) + ".")
vlist = AIVehicleList_Station(stop)

// remove non-rvs
foreach (v,z in vlist) {
//AILog.Info("ping")
if ((AIVehicle.GetVehicleType(v) != AIVehicle.VT_ROAD)) {
vlist.RemoveItem(v);
}
}
// remove non-buses
rvlist.Valuate(AIVehicle.GetCapacity, 0);
rvlist.KeepAboveValue(9);
 
if (vlist.Count() < 30) {	// don't build more in saturated stops	

foreach (bus, v in vlist) {		

// remove coaches from the list, to bias towards adding local buses
local order1 = AIOrder.GetOrderDestination(bus, 0)
local order2 = AIOrder.GetOrderDestination(bus, 1)

if ((AITile.GetClosestTown(order1) != AITile.GetClosestTown(order2))
//&& (AIBase.RandRange(4) == 0) 
&& (vlist.Count() > 1)) {
vlist.RemoveItem(v); 
//AILog.Info("pang")
}
}

vlist.Valuate(AIBase.RandItem); // shuffle the list
obus = vlist.Begin();

// compare orders; if first two orders are in different towns, build a coach instead of a bus
local order1 = AIOrder.GetOrderDestination(obus, 0)
local order2 = AIOrder.GetOrderDestination(obus, 1)

if (AITile.GetClosestTown(order1) == AITile.GetClosestTown(order2)) {
upbus = IdentifyBus(true);
} else {
upbus = IdentifyCoach(true);
}


if (AIVehicle.IsValidVehicle(obus)) {
clonebus = AIVehicle.BuildVehicle(HomeDepot,upbus-1)
if (AIVehicle.IsValidVehicle(clonebus)) {
AILog.Info("I've bought another vehicle for " + AITown.GetName(AIStation.GetNearestTown(stop)) + ".")
AIOrder.ShareOrders(clonebus,obus);
local nx = 0
local name = null
local changename = false

while (!changename && (nx < 1000)) {
nx++ 
name = ((AITown.GetName(AIStation.GetNearestTown(stop))) + " " + nx)
changename = AIVehicle.SetName(clonebus,name)
}
AIOrder.SkipToOrder(clonebus,AIBase.RandRange(6));
AIGroup.MoveVehicle(Groups[0], clonebus); // add to appropriate group
if (AIOrder.IsValidVehicleOrder(clonebus, 1)) {// Since the mail truck update, occasionally we built an intercity bus with no orders. I could find the bug, but easier just to quietely sell the vehicles back.
AIVehicle.StartStopVehicle(clonebus);
} else {
AIVehicle.SellVehicle(clonebus);
AILog.Info("Oops");
}
}
} else {
AILog.Info("For some reason (probably not enough money), I couldn't buy a vehicle for " + AITown.GetName(AIStation.GetNearestTown(stop)) + ".")
}	
} else {
// AILog.Info("I won't buy another vehicle for " + (AIBaseStation.GetName(stop)) + ", because it's already crowded.")
}
}
}
} else {
AILog.Info("I've already got my maximum number of buses, so I won't build more.")
}
} else {
AILog.Info("I don't have the money for new buses at the moment.")
}


// do we want to remove unprofitable buses?

local rvlist = AIVehicleList();
foreach (veh,z in rvlist) {
if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_ROAD) {
rvlist.RemoveItem(veh);
}
}

for (local v = rvlist.Begin(); !(rvlist.IsEnd()); v = rvlist.Next()) {
if  ((AIVehicle.GetProfitLastYear(v) < 0) &&
	(AIVehicle.GetProfitThisYear(v) < 0) &&
//	(AIVehicle.GetCapacity(v,0) > 9) &&
	(AIVehicle.GetAge(v) > (365*2))) {
	AIVehicle.SendVehicleToDepot(v);
	AILog.Info(AIVehicle.GetName(v) + " is losing money, so I'm sending it to the depot.")	
	}
}

return
}


// ====================================================== 
//               CLEAN VEHICLES FROM DEPOT
// ====================================================== 

function CivilAI::DepotClean() {

AILog.Info("I'm clearing out old vehicles from my depots.");

local rvlist = AIVehicleList();
local c = 0;

for (local v = rvlist.Begin(); !(rvlist.IsEnd()); v = rvlist.Next()) {
if  (AIVehicle.IsStoppedInDepot(v)) {
	AIVehicle.SellVehicle(v);
	c++	
	}
	}
if (c == 0) {	
AILog.Info("I found nothing to sell.")
} else if (c == 1) {	
AILog.Info("I've disposed of " + c + " vehicle.")
} else {
AILog.Info("I've disposed of " + c + " vehicles.")
}

return	
}

// ====================================================== 
//               BUILD ADDITIONAL NETWORKS
// ====================================================== 

function CivilAI::NewNetwork() {

local dosh = AICompany.GetBankBalance(Me);
local nbus = IdentifyBus(false);

if (nbus == false) {
return;
}
else if (dosh > (AIEngine.GetPrice(nbus-1) * 3)) {

// check we don't have MaxBus
local rvlist = AIVehicleList();
foreach (veh,z in rvlist) {
if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_ROAD) {
rvlist.RemoveItem(veh);
}
}
rvlist.Valuate(AIVehicle.GetCapacity, 0);
rvlist.KeepAboveValue(9);
local buscount = rvlist.Count();
if (buscount < MaxBus) {

AILog.Info("I'm looking for a town to build a new bus network in.");

// find a town

local townlist = MyTownList();
townlist.Valuate(AITown.GetPopulation);
townlist.RemoveBelowValue(MinPop);

foreach (town,z in townlist) {
if ((AITown.GetLastMonthSupplied(town, 0) * 20) > AITown.GetPopulation(town)) {
townlist.RemoveItem(town);  // discard sufficiently serviced towns
//AILog.Info("Discarding " + AITown.GetName(town) + " as sufficiently serviced.");

// build a depot anyway for our towncars
local dlist = AIDepotList(AITile.TRANSPORT_ROAD); 
foreach (depot,z in dlist) {
	if (AITile.GetClosestTown(depot) != town) {
		dlist.RemoveItem(depot);	
		}
	}
if (dlist.Count() == 0) {
NewDepot(town, false);
//AILog.Info("Building a depot only (for towncars) in " + AITown.GetName(town) + ".");
}

} else {
local bslist = AIStationList(AIStation.STATION_BUS_STOP);
foreach (stat,z in bslist) {
if (AIStation.GetNearestTown(stat) == (town)) {
townlist.RemoveItem(town);  // discard self-serviced towns
//AILog.Info("Discarding " + AITown.GetName(town) + " as self-serviced.");
break;
}
}
}
}

if (townlist.Count() > 0) {
townlist.Valuate(AIBase.RandItem); // shuffle the town list
local newtown = townlist.Begin();

// build a new network. First, check for an existing depot


local dlist = AIDepotList(AITile.TRANSPORT_ROAD); 
foreach (depot,z in dlist) {
	if (AITile.GetClosestTown(depot) != newtown) {
		dlist.RemoveItem(depot);	
		}
	}
if (dlist.Count() == 0) {
NewDepot(newtown, true);
} else {
BuildBuses(newtown,dlist.Begin());
}

} else {
AILog.Info("I couldn't find one.");
}
} else {
AILog.Info("I've already got my maximum number of buses, so I won't build a new network.")
}
} else {
AILog.Info("I don't have the money for a new network at the moment.")
}
return
}

// ====================================================== 
//          BUILD ADDITIONAL DEPOT FOR NETWORK
// ====================================================== 

function CivilAI::NewDepot(town, buildbuses) {

// clockwise spiral out
local trytilegrid = [AIMap.GetTileX(AITown.GetLocation(town)),AIMap.GetTileY(AITown.GetLocation(town))]
local trytile;
local x = 1
local y = 1
local i = 5
local depot = 0
while ((depot == 0) && (i < 20)) {
//y--
for (;y >= 0-i;y--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, "?");
//AILog.Info("testing tile at " + x + "," + y);
if (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD) 
&& !AIRoad.IsRoadDepotTile(trytile)
&& !AIRoad.IsRoadStationTile(trytile)
&& !AIRoad.IsDriveThroughRoadStationTile(trytile)
&& !AIBridge.IsBridgeTile(trytile)
&& !depot)
depot = BuildDepot(trytile,town);
}
//x--;
for (;x >= 0-i;x--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, "?");
//AILog.Info("testing tile at " + x + "," + y);
if (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD) 
&& !AIRoad.IsRoadDepotTile(trytile)
&& !AIRoad.IsRoadStationTile(trytile)
&& !AIRoad.IsDriveThroughRoadStationTile(trytile)
&& !AIBridge.IsBridgeTile(trytile)
&& !depot)
depot = BuildDepot(trytile,town);
}
//y++;
for (;y <= i;y++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, "?");
//AILog.Info("testing tile at " + x + "," + y);
if (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD)
&& !AIRoad.IsRoadDepotTile(trytile)
&& !AIRoad.IsRoadStationTile(trytile)
&& !AIRoad.IsDriveThroughRoadStationTile(trytile)
&& !AIBridge.IsBridgeTile(trytile)
&& !depot)
depot = BuildDepot(trytile,town);
}
//x++;
for (;x <= i;x++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, "?");
//AILog.Info("testing tile at " + x + "," + y);
if (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD)
&& !AIRoad.IsRoadDepotTile(trytile)
&& !AIRoad.IsRoadStationTile(trytile)
&& !AIRoad.IsDriveThroughRoadStationTile(trytile)
&& !AIBridge.IsBridgeTile(trytile)
&& !depot)
depot = BuildDepot(trytile,town);
}
i=i+1
}

if (depot) {
AILog.Info("I've built a new depot in " + AITown.GetName(town) + ".");
if (buildbuses) {
BuildBuses(town,depot);
}
} else {
AILog.Info("I couldn't find a suitable location for a depot in " + AITown.GetName(town) + ".");
return
}
}

//<
// ====================================================== 
// ====================================================== 
//         RRRRR     OOOO      AAAA     DDDD
//         R   RR   O    O    A    A    D  DD
//         RRRRR   OO    OO   AAAAAA    D   DD
//         R   R    O    O    A    A    D  DD
//         R    R    OOOO     A    A    DDDD
// ====================================================== 
// ====================================================== 
//>
// ====================================================== 
//                  CHOOSE ROAD CONNECTION
// ====================================================== 

function CivilAI::MappaMundi() {

local a
local b

//AILog.Info("I'm connecting towns with road.");

// new check: have we built in most of the towns already connected? If not, don't waste money expanding.

local zoop = MyTownList();

zoop.Valuate(AITown.GetPopulation);
zoop.RemoveBelowValue(MinPop); // only buildable towns considered

local yoop = AIDepotList(AITile.TRANSPORT_ROAD); 

//AILog.Info("I currently have " + yoop.Count() + " depots in " + zoop.Count() + " towns.");

if (zoop.Count() < yoop.Count() + 2) {

local hq = AICompany.GetCompanyHQ(Me);
local townlist = AITownList();

townlist.Valuate(AITown.GetPopulation);
townlist.RemoveBelowValue(MinPop); // don't connect to towns we can't build in
townlist.AddItem(AITile.GetClosestTown(hq), 0); // add back the home town, in case it's small
townlist.AddList(IndTownList); // but add back any we want to connect for industrial reasons

 // sort by distance from the hq
townlist.Valuate(AITown.GetDistanceManhattanToTile, hq);


townlist.KeepBelowValue(NetworkRadius); // remove towns which are too far away
townlist.RemoveList(Dudtowns); // remove known unconnectables

townlist.Sort(AIList.SORT_BY_VALUE,true);

// add the home town's home tile to the target list
local hometown = townlist.Begin();
local mapnodes = [AITown.GetLocation(hometown)] 
local nodecount = 1
local target = -1

foreach (town,z in townlist) {

local testroad = RoadPF();
local towntile = AITown.GetLocation(town);
if (!AITile.HasTransportType(towntile,AITile.TRANSPORT_ROAD) && (AITown.GetRoadReworkDuration(town) == 0)) {
Dudtowns.AddItem(town,0) // add this town to a list of unreachables
AILog.Info(AITown.GetName(town) + " has no central road tile.")
} else {
testroad.cost.no_existing_road = testroad.cost.max_cost; // only check existing roads
testroad.InitializePath([towntile], mapnodes);

local path = false;
while (path == false) {
path = testroad.FindPath(50);
AIController.Sleep(1);
}

if (path != null) { 			// we are connected already
mapnodes.append(towntile); // add this town to the nodes we can connect to
nodecount ++
// AILog.Info(AITown.GetName(town) + " is connected.")
} else {
target = town;
break;
}
}
}

if (target != -1) {
a = [AITown.GetLocation(target)]
b = mapnodes

BuildARoad(a,b,target)
} else {
AILog.Info("I have connected all available towns to the road network.")
}
} // end zoop
else {
AILog.Info("I won't expand my network just yet.")
}

// load the list of known connected towns for the review
local townlist = MyTownList();

local rc = 0;
local ThisMonth = AIDate.GetMonth(AIDate.GetCurrentDate());
local ReviewMonth = AIDate.GetMonth(AIDate.GetCurrentDate());

while ((rc < ((townlist.Count() / 4) + 1)) && (ThisMonth == ReviewMonth)) { 
local revdist = 1000000;
local x = 0
local sstowna
local sstownb

while ((revdist > 128) && (x < 100)) { // was ((NetworkRadius / 2) + 64)
townlist.Valuate(AIBase.RandItem); // shuffle the town list
sstowna = townlist.Begin();
sstownb = townlist.Next();
a = [AITown.GetLocation(sstowna)]
b = [AITown.GetLocation(sstownb)]
revdist = AITile.GetDistanceManhattanToTile(a[0],b[0])
x++
}
BuildARoad(a,b,-1)
rc++
ThisMonth = AIDate.GetMonth(AIDate.GetCurrentDate());
}
return;
}

// ====================================================== 
//                  BUILD ROAD CONNECTION
// ====================================================== 

function CivilAI::BuildARoad(a,b,target) {

if (target != -1) {
// check funds allow for the connection, and a bit left over to maintain the network
// as of 1.6 we only do this check for building new links, because we want to always check eg after building bridges
local condist = NetworkRadius
local nodedist
foreach (node, loc in b) {
nodedist = AITile.GetDistanceManhattanToTile(a[0],loc)
if (nodedist < condist) condist = nodedist
}

local dosh = AICompany.GetBankBalance(Me);
if ((dosh) < (AIRoad.GetBuildCost(AIRoad.ROADTYPE_ROAD, AIRoad.BT_ROAD) * 2 * condist)) {
if (target != -1) {
AILog.Info("I can't afford to build a road to " + AITown.GetName(target)  + " right now. Perhaps later.")
}

return;
}	
AILog.Info("I'm connecting " + AITown.GetName(target) + " to the road network.")
} else {
AILog.Info("I'm reviewing the roads between " + AITown.GetName(AITile.GetClosestTown(a[0])) + " and " + AITown.GetName(AITile.GetClosestTown(b[0])) + ".")
}

local buildroad = RoadPF();
buildroad.cost.max_cost 			= 10000000;   //10000000;
buildroad.cost.tile     			= 100;        // 100;
buildroad.cost.no_existing_road		= 75;        //40;
buildroad.cost.turn           		= 100;        //100;
buildroad.cost.slope          		= 400;        //200;
buildroad.cost.bridge_per_tile		= 250;        //150;
buildroad.cost.tunnel_per_tile		= 500;        //120;
buildroad.cost.coast       			= 100;        //20;
buildroad.cost.max_bridge_length	= 10;          //10;
buildroad.cost.max_tunnel_length	= 20;          //20;

buildroad.InitializePath(a, b);

local path = false;
while (path == false) {
  path = buildroad.FindPath(50);
  AIController.Sleep(1);
}


if (path == null) {

Dudtowns.AddItem(target,0) // add this town to a list of unreachables
AILog.Info(AITown.GetName(target) + " seems to be unreachable.")

//
return
}

local c = 0


while (path != null) {
  local par = path.GetParent();
  if (par != null) {
    local last_node = path.GetTile();
    if (AIMap.DistanceManhattan(path.GetTile(), par.GetTile()) == 1 ) {
      if ((!AIRoad.BuildRoad(path.GetTile(), par.GetTile())) && c < 10) {
			/* An error occured while building a road. TODO: handle it. */
      }
    } else {
      /* Build a bridge or tunnel. */
      if (!AIBridge.IsBridgeTile(path.GetTile()) && !AITunnel.IsTunnelTile(path.GetTile())) {
        /* If it was a road tile, demolish it first. Do this to work around expended roadbits. */
        if (AIRoad.IsRoadTile(path.GetTile())) AITile.DemolishTile(path.GetTile());
        if (AITunnel.GetOtherTunnelEnd(path.GetTile()) == par.GetTile()) {
          if (!AITunnel.BuildTunnel(AIVehicle.VT_ROAD, path.GetTile())) {
            /* An error occured while building a tunnel. TODO: handle it. */
          }
        } else {
          local bridge_list = AIBridgeList_Length(AIMap.DistanceManhattan(path.GetTile(), par.GetTile()) + 1);
          bridge_list.Valuate(AIBridge.GetMaxSpeed);
          bridge_list.Sort(AIList.SORT_BY_VALUE, false);
          if (!AIBridge.BuildBridge(AIVehicle.VT_ROAD, bridge_list.Begin(), path.GetTile(), par.GetTile())) {
            /* An error occured while building a bridge. TODO: handle it. */
          }
        }
      }
    }
  }
  path = par;
}
//AILog.Info("I've finished road building for now.")

return
}

// ====================================================== 
//    FIND LEVEL XINGS TO REPLACE,Also Review Roads
// ====================================================== 

function CivilAI::XingReplace() {
while (AIEventController.IsEventWaiting()) {
local event = AIEventController.GetNextEvent();

if (event.GetEventType() == AIEvent.ET_VEHICLE_CRASHED) {
	local crash = AIEventVehicleCrashed.Convert(event);
		if (crash.GetCrashReason() == AIEventVehicleCrashed.CRASH_RV_LEVEL_CROSSING) {
			local xing = crash.GetCrashSite();
			local xinggrid = [AIMap.GetTileX(xing),AIMap.GetTileY(xing)]			
			AILog.Info("One of our vehicles got squished at " + AIMap.GetTileX(xing) + "," + AIMap.GetTileY(xing) + ". I'll remove the road if I can."); 


AIRoad.RemoveRoad(AIMap.GetTileIndex(xinggrid[0]-1,xinggrid[1]), AIMap.GetTileIndex(xinggrid[0]+1,xinggrid[1]))
AIRoad.RemoveRoad(AIMap.GetTileIndex(xinggrid[0],xinggrid[1]-1), AIMap.GetTileIndex(xinggrid[0],xinggrid[1])+1)
			
			}
		} else if (event.GetEventType() == AIEvent.ET_VEHICLE_LOST) {
	local losy = AIEventVehicleLost.Convert(event);
	local lost = losy.GetVehicleID();
	
if (AIVehicle.GetVehicleType(lost) == AIVehicle.VT_ROAD) {	
local a = [AIVehicle.GetLocation(lost)]
local b = [AIOrder.GetOrderDestination(lost, AIOrder.ORDER_CURRENT)]

AILog.Info ("Checking roads for lost vehicle " + AIVehicle.GetName(lost) + ".")

BuildARoad(a,b,-1);
}
}
}
AILog.Info ("No reported network issues.")
}

//<
// ====================================================== 
// ====================================================== 
//         CCCC     AAAA    RRRRR     SSSS
//        C    C   A    A   R   RR   SS   S
//       CC        AAAAAA   RRRRR      SS
//        C    C   A    A   R   R    S   SS
//         CCCC    A    A   R    R    SSSS
// ====================================================== 
// ====================================================== 
//>
// ====================================================== 
//                 MANAGE TOWN CARS
// ====================================================== 

function CivilAI::Vroom() {

// do we have the maximum cars?

local rvlist = AIVehicleList();
foreach (veh,z in rvlist) {
if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_ROAD) {
rvlist.RemoveItem(veh);
}
}
rvlist.Valuate(AIVehicle.GetCapacity, 0);
rvlist.KeepBelowValue(10);
local carcount = rvlist.Count();
if (carcount < MaxCar) {


local vlist = AIEngineList(AIVehicle.VT_ROAD); 	// list of all road vehicles

// valid town cars have no running cost and < 10 capacity.

vlist.Valuate(AIEngine.IsValidEngine);
vlist.KeepValue(1);
vlist.Valuate(AIEngine.GetRunningCost);
vlist.KeepValue(0);
vlist.Valuate(AIEngine.GetCapacity);
vlist.KeepBelowValue(10);

if (vlist.Count() == 0) {
AILog.Info("I can't find any town cars available to buy...")

return
} else {

local townlist = MyTownList();

// ====================================================== 
//      calculate the maximum number of cars to buy
// ====================================================== 

// the maximum number of towncars is governed by 3 factors;
// the config setting.
// the ratio of unserved passengers.
// the total population of the network.

// get total population
local population = 0.0
foreach (town,z in townlist) {
population = (population + AITown.GetPopulation(town))
}
// get passengers produced
local totpax = 1 // add 1, just in case something goes wrong and we end up dividing by 0.
foreach (town,z in townlist) {
totpax = (totpax + AITown.GetLastMonthProduction(town, 0))  // again assuming pax = cargo 0
}
// get passengers carried
local transpax = 0
foreach (town,z in townlist) {
transpax = (transpax + (AITown.GetLastMonthProduction(town, 0) * AITown.GetLastMonthTransportedPercentage(town, 0)))
}

local unservedperc = (100 - (transpax / totpax))

local mapsize = ((AIMap.GetMapSizeX() + AIMap.GetMapSizeY() ) / 2)
if (mapsize > NetworkRadius) { mapsize = NetworkRadius }
local worldsize = ((mapsize / 64) * (mapsize / 64))


local MaxToBuild = ((MaxCar * unservedperc / 100) * (population / worldsize / 16000) * 2).tointeger();
if (MaxToBuild > MaxCar) { MaxToBuild = MaxCar }

local CarsToBuild = (MaxToBuild - carcount)

if (CarsToBuild < 1) {
AILog.Info("No drivers want new cars (" + carcount + " town cars active).")
} else if (CarsToBuild == 1) {
AILog.Info(CarsToBuild + " driver wants a new car (" + carcount + " town cars active).")
} else {
AILog.Info(CarsToBuild + " drivers want new cars (" + carcount + " town cars active).")
//AILog.Info(CarsToBuild + " drivers want new cars (" + (((population / worldsize / 160).tointeger()) + 1) +  "% population density, " + (100 - unservedperc) + "% of passengers transported, " + carcount + " town cars active).")
}
// ====================================================== 
//             purchase and name town cars
// ====================================================== 

if (CarsToBuild > BuyCar) { CarsToBuild = BuyCar }
local c = 0
local newcar


local dlist = AIDepotList(AITile.TRANSPORT_ROAD); 	

local namefirst = ["Jim", "Bob", "Harry", "Charlie", "Eric", "Owen", "Hans", "Keith",
				   "Mike", "Albert", "Gordon", "Dave", "Clive", "Olaf", "Anthony", "Bruce",
 				   "Kelly", "Susan", "Jessie", "Jane", "Sally", "Jean", "Ruth", "Elizabeth",
				   "Mary", "Anne", "Casey", "Kate", "Ingrid", "Lucy", "Colleen", "Julia",
				   "Gwyneth", "Sam", "Margaret", "Antonia", "Gerald", "Donald", "Colin", "Dennis",
				   "Basil", "Norma", "Jeremy", "Virginia", "Emma", "Stanley", "Gladys", "Tim"]

local namelast = ["Spriggs", "Hudson", "Simpson", "McDonald", "Clarkson", "Robertson", "Jones", "Griffith",
				  "Mills", "Palmer", "Morecambe", "Rudge", "Wells", "Evans", "Dixon", "Williams",
				  "Wood", "Foster", "Jackson", "O'Malley", "Lewis", "Deakin", "Mackintosh", "Talbot",
				  "Petersen", "Churchill", "Worthington", "Steel", "Boyd", "McGrath", "Hilton", "Smith",
				  "Hughes", "Horner", "Jones", "Brown", "Dalziel", "Menzies", "Baxter", "Bond",
				  "MacArthur", "Mayne", "Armstrong", "Waterman", "Freud", "Abbott", "Cooper", "James"]
				  

while (c < CarsToBuild) {
dlist.Valuate(AIBase.RandItem); // shuffle the list
vlist.Valuate(AIBase.RandItem); // shuffle the list
newcar = (AIVehicle.BuildVehicle(dlist.Begin(),vlist.Begin())); 
if (AIVehicle.IsValidVehicle(newcar)) {
local nx = 0
local name = null
local changename = false
while (!changename && (nx < 1000)) {
nx++ 
name = (namefirst[AIBase.RandRange(48)] + " " + namelast[AIBase.RandRange(48)])
changename = AIVehicle.SetName(newcar,name)
}
AILog.Info("I've bought a car for " + AIVehicle.GetName(newcar) + ".")
AIGroup.MoveVehicle(Groups[1], newcar); // add to appropriate group
AIVehicle.StartStopVehicle(newcar);
} else {
AILog.Info("For some reason (probably not enough money), I couldn't buy a new car.")
}
c++
}
}

} else {
AILog.Info("I've already got my maximum number of town cars, so I won't build more.")
}

// ====================================================== 
//             send old cars to the depot
// ====================================================== 

for (local v = rvlist.Begin(); !(rvlist.IsEnd()); v = rvlist.Next()) {
if  ((AIVehicle.GetAgeLeft(v) < (365 * 1)) && !(AIVehicle.IsStoppedInDepot(v)) && AIVehicle.IsValidVehicle(v)) {
AIVehicle.SendVehicleToDepot(v);
//AILog.Info(AIVehicle.GetName(v) + "'s car is getting old, so I'm sending it to the depot.")
}
}

return
}

// ====================================================== 
//            		 Group Management
// ====================================================== 

function CivilAI::MakeGroups() {

Groups[0] = AIGroup.CreateGroup(AIVehicle.VT_ROAD);
Groups[1] = AIGroup.CreateGroup(AIVehicle.VT_ROAD);
Groups[2] = AIGroup.CreateGroup(AIVehicle.VT_ROAD);
Groups[3] = AIGroup.CreateGroup(AIVehicle.VT_ROAD);

if (AIGroup.SetName(Groups[0], "Buses")) {} else { AIGroup.SetName(Groups[0], "Buses " + Me ) }
if (AIGroup.SetName(Groups[1], "Cars")) {} else { AIGroup.SetName(Groups[1], "Cars " + Me ) }
if (AIGroup.SetName(Groups[2], "Mail")) {} else { AIGroup.SetName(Groups[2], "Mail " + Me ) }
if (AIGroup.SetName(Groups[3], "Goods")) {} else { AIGroup.SetName(Groups[3], "Goods " + Me ) }
}

function CivilAI::LoadGroups() {

local glist = AIGroupList();
foreach (group,z in glist) {

	 if (AIGroup.GetName(group) == "Buses" || AIGroup.GetName(group) == "Buses " + Me) { Groups[0] = group; } 
else if (AIGroup.GetName(group) == "Cars" || AIGroup.GetName(group) == "Cars " + Me) { Groups[1] = group; } 
else if (AIGroup.GetName(group) == "Mail" || AIGroup.GetName(group) == "Mail " + Me) { Groups[2] = group; } 
else if (AIGroup.GetName(group) == "Goods" || AIGroup.GetName(group) == "Goods " + Me) { Groups[3] = group; } 
}
}


//<
// ====================================================== 
// ====================================================== 
//         TTTTTT    OOOO    W     W   N   N
//           TT     O    O   WW   WW   NN  N
//           TT    OO    OO   W W W    N N N
//           TT     O    O    WWWWW    N  NN
//           TT      OOOO      W W     N   N
// ====================================================== 
// ====================================================== 
//>
// ====================================================== 
//                  LIST CONNECTED TOWNS
// ====================================================== 

function CivilAI::MyTownList() {

local hq = AICompany.GetCompanyHQ(Me);
local townlist = AITownList();
// sort by distance from the hq
townlist.Valuate(AITown.GetDistanceManhattanToTile, hq);
townlist.KeepBelowValue(NetworkRadius); // remove towns which are too far away
townlist.RemoveList(Dudtowns); // remove known unconnectables

townlist.Sort(AIList.SORT_BY_VALUE,true);

// add the home town's home tile to the target list
local hometown = townlist.Begin();
local mapnodes = [AITown.GetLocation(hometown)] 
local nodecount = 1
local target = -1

foreach (town,z in townlist) {

local testroad = RoadPF();
local towntile = AITown.GetLocation(town);
testroad.cost.no_existing_road = testroad.cost.max_cost; // only check existing roads
testroad.InitializePath([towntile], mapnodes);

local path = false;
while (path == false) {
path = testroad.FindPath(50);
AIController.Sleep(1);
}

if (path != null) { 			// we are connected already
mapnodes.append(towntile); // add this town to the nodes we can connect to
nodecount ++
// AILog.Info(AITown.GetName(town) + " is connected.")
} else {
townlist.RemoveItem(town);  // discard disconnected towns
}
}

// our townlist now contains only connected towns within the network radius.

return townlist;
}



//<
// ====================================================== 
// ====================================================== 
// 			   AAAA  	  II	  RRRRR  
// 			  A    A 	  II	  R   RR 
// 			  AAAAAA 	  II	  RRRRR  
//  		  A    A 	  II	  R   R  
//  		  A    A 	  II	  R    R 
// ====================================================== 
// ====================================================== 
//>
// ====================================================== 
//                   BUILD AIRPORT
// ====================================================== 

function CivilAI::Airportz() {
if ((AIAirport.IsValidAirportType(AIAirport.AT_SMALL) || 
	 AIAirport.IsValidAirportType(AIAirport.AT_COMMUTER)) &&
	 (AIGameSettings.IsDisabledVehicleType(AIVehicle.VT_AIR) == false))
	 
	 {
	 
	 
local dosh = AICompany.GetBankBalance(Me);
if (dosh > (AIAirport.GetPrice(AIAirport.AT_COMMUTER) * 4)) {	 
AILog.Info("Looking for an airport site...")	


local aplist = AIStationList(AIStation.STATION_AIRPORT); 
local bslist = AIStationList(AIStation.STATION_BUS_STOP); 
local airdist;

bslist.Valuate(AIStation.GetCargoWaiting, 0); //order by pax waiting
bslist.RemoveBelowValue(250);

foreach (stop,z in bslist) {
if ((AITown.GetAllowedNoise(AIStation.GetNearestTown(stop))) < 4) {
bslist.RemoveItem(stop);		// remove towns with insufficent noise allowance
}
}

// now added in 1.6 - don't build airports too close to all existing airports.
// I stuffed this up in 1.7 - now it doesn't build too close to *any* existing airport. Might still be a good change?

if (aplist.Count() > 0) {
foreach (airport,z in aplist) { // remove stops in serviced towns, or too close to existing airports
foreach (stop,z in bslist) {
if ((AIStation.GetNearestTown(stop)) == (AIStation.GetNearestTown(airport))) {
bslist.RemoveItem(stop);
}
airdist = AIMap.DistanceManhattan(AIBaseStation.GetLocation(stop),AIBaseStation.GetLocation(airport));
if (airdist < MinAirRange * 0.5) {
bslist.RemoveItem(stop);
}
}
}
}

if (bslist.Count() < 1) {
AILog.Info("I couldn't find one.");
} else {
bslist.Valuate(AIBase.RandItem); // shuffle the list
local newapsite = bslist.Begin();

// check town approval before levelling mountains!
//AILog.Info(AITown.GetRating(AIStation.GetNearestTown(newapsite), Me))
if (AITown.GetRating(AIStation.GetNearestTown(newapsite), Me) < 4) {
return;
}


AILog.Info("attempting to build an airport at " + AIBaseStation.GetName(newapsite) + "."); 


BuildAirport(newapsite,AIBaseStation.GetLocation(newapsite)); // try to build an airport attached to this bus stop

} 
	 } else {
AILog.Info("I can't afford an airport right now.")
}
	 
	 } else {
AILog.Info("I can't build an airport right now.")
}

return
}

// ====================================================== 
//                   SPIRAL AIRPORT
// ====================================================== 
function CivilAI::BuildAirport(station,location) {
// clockwise spiral out
local trytilegrid = [AIMap.GetTileX(location),AIMap.GetTileY(location)]
local trytile;
local x = 0
local y = 0
local i = 0
local s = AIGameSettings.GetValue("station.station_spread")

// commuter

if (AIAirport.IsValidAirportType(AIAirport.AT_COMMUTER)) {

while ((i < s) && !(AIStation.HasStationType(station,AIStation.STATION_AIRPORT))) {
//y--
for (;y >= 0-i;y--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if (AITile.IsBuildableRectangle(trytile, 5, 4)) {
BuildComAp(station,trytile);
}
} 
//x--;
for (;x >= 0-i;x--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if (AITile.IsBuildableRectangle(trytile, 5, 4)) {
BuildComAp(station,trytile);
}
} 
//y++;
for (;y <= i;y++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if (AITile.IsBuildableRectangle(trytile, 5, 4)) {
BuildComAp(station,trytile);
}
} 
//x++;
for (;x <= i;x++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if (AITile.IsBuildableRectangle(trytile, 5, 4)) {
BuildComAp(station,trytile);
}
} 
i=i+1
}
if (i == s) {
AILog.Info("I failed to find an airport site at " + AIBaseStation.GetName(station) + "."); 
}
} else if (AIAirport.IsValidAirportType(AIAirport.AT_SMALL)) {

while ((i < s) && !(AIStation.HasStationType(station,AIStation.STATION_AIRPORT))) {
//y--
for (;y >= 0-i;y--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if (AITile.IsBuildableRectangle(trytile, 4, 3)) {
BuildSmAp(station,trytile);
}
} 
//x--;
for (;x >= 0-i;x--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if (AITile.IsBuildableRectangle(trytile, 4, 3)) {
BuildSmAp(station,trytile);
}
} 
//y++;
for (;y <= i;y++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if (AITile.IsBuildableRectangle(trytile, 4, 3)) {
BuildSmAp(station,trytile);
}
} 
//x++;
for (;x <= i;x++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if (AITile.IsBuildableRectangle(trytile, 4, 3)) {
BuildSmAp(station,trytile);
}
} 
i=i+1
}
if (i == s) {
AILog.Info("I failed to find an airport site at " + AIBaseStation.GetName(station) + "."); 
}
}
}

function CivilAI::BuildComAp(station,location) {

// level the land as a matter of course now
// flatten land from the center

local tg = [AIMap.GetTileX(location),AIMap.GetTileY(location)]
local ct = AIMap.GetTileIndex(tg[0]+2,tg[1]+2)

AITile.LevelTiles(ct,AIMap.GetTileIndex(tg[0],tg[1]));
AITile.LevelTiles(ct,AIMap.GetTileIndex(tg[0],tg[1]+4));
AITile.LevelTiles(ct,AIMap.GetTileIndex(tg[0]+5,tg[1]));
AITile.LevelTiles(ct,AIMap.GetTileIndex(tg[0]+5,tg[1]+4));

if (AIAirport.BuildAirport(location,AIAirport.AT_COMMUTER,station)) {
AILog.Info("Built a commuter airport.");
AirAssign(station);
return true;
}
}

function CivilAI::BuildSmAp(station,location) {

// level the land as a matter of course now
// flatten land from the center

local tg = [AIMap.GetTileX(location),AIMap.GetTileY(location)]
local ct = AIMap.GetTileIndex(tg[0]+2,tg[1]+1)

AITile.LevelTiles(ct,AIMap.GetTileIndex(tg[0],tg[1]));
AITile.LevelTiles(ct,AIMap.GetTileIndex(tg[0],tg[1]+3));
AITile.LevelTiles(ct,AIMap.GetTileIndex(tg[0]+4,tg[1]));
AITile.LevelTiles(ct,AIMap.GetTileIndex(tg[0]+4,tg[1]+3));

if (AIAirport.BuildAirport(location,AIAirport.AT_SMALL,station)) {
AILog.Info("Built a small airport.");
AirAssign(station);
return true;
}
}

// ====================================================== 
//                   BUILD AIR ROUTES
// ====================================================== 
function CivilAI::AirReview() {

local aplist = AIStationList(AIStation.STATION_AIRPORT);

// remove saturated airports
foreach (airport,z in aplist) { 

local vlist = AIVehicleList_Station(airport)
foreach (veh,z in vlist) {
if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_AIR) {
vlist.RemoveItem(veh);
}
}

if (((AIAirport.GetAirportType(AIBaseStation.GetLocation(airport)) == AIAirport.AT_SMALL) && vlist.Count() < 3 ) || ((AIAirport.GetAirportType(AIBaseStation.GetLocation(airport)) == AIAirport.AT_COMMUTER) && vlist.Count() < 6 )) {
} else {
aplist.RemoveItem(airport)
}
}

if (aplist.Count() > 1) {

for (local c = 0; c < BuyPlane; c++) {

aplist.Valuate(AIBase.RandItem); // shuffle the airport list
local ap1 = aplist.Begin();
local ap2 = aplist.Next();

local airdist = AIMap.DistanceManhattan(AIBaseStation.GetLocation(ap1),AIBaseStation.GetLocation(ap2));

if (airdist > MinAirRange) {

AILog.Info("Reviewing air service between " + AIBaseStation.GetName(ap1) + " and " + AIBaseStation.GetName(ap2) + " (" + airdist + " tiles)."); 

local paxwait = (AIStation.GetCargoWaiting(ap1,0) + AIStation.GetCargoWaiting(ap2,0))

// select an aircraft

local acmodels = AIEngineList(AIVehicle.VT_AIR);

// remove unsuitables

local ranger = AIOrder.GetOrderDistance(AIVehicle.VT_AIR, AIBaseStation.GetLocation(ap1), AIBaseStation.GetLocation(ap2))

// AILog.Info(paxwait + " passengers waiting, and distance is " + ranger + "."); 

foreach (ac, z in acmodels) {
if ((AIEngine.GetPlaneType(ac) != AIAirport.PT_SMALL_PLANE) && (AIEngine.GetPlaneType(ac) != AIAirport.PT_HELICOPTER)) 
{ 
// AILog.Info("Discarding " + AIEngine.GetName(ac) + " because it is not a small aircraft."); 
acmodels.RemoveItem(ac)
}

if ((AIEngine.GetMaximumOrderDistance(ac) != 0) && (AIEngine.GetMaximumOrderDistance(ac) < ranger))
{ 
// AILog.Info("Discarding " + AIEngine.GetName(ac) + " because it has insufficient range."); 
acmodels.RemoveItem(ac)
}

if (AIEngine.GetCargoType(ac) != 0)
{ 
// AILog.Info("Discarding " + AIEngine.GetName(ac) + " because it doesn't carry Passengers."); 
acmodels.RemoveItem(ac)
}

if (AIEngine.GetCapacity(ac) > (paxwait / 10))
{ 
// AILog.Info("Discarding " + AIEngine.GetName(ac) + " because it is bigger than required."); 
acmodels.RemoveItem(ac)
}

local dosh = AICompany.GetBankBalance(Me);
if (AIEngine.GetPrice(ac) > (dosh * 1))
{ 
// AILog.Info("Discarding " + AIEngine.GetName(ac) + " because of insufficient funds."); 
acmodels.RemoveItem(ac)
}


}

local acid= 0;
local blist= 0;
local bcap = 0;
local bdate= 0;
local z = 0;

// now pick one

foreach (ac,z in acmodels) {

//AILog.Info("Assessing " + AIEngine.GetName(ac));

if (AIEngine.IsBuildable(ac))  {

if (AIEngine.GetCapacity(ac) > bcap) {				    // highest capacity.
 acid = ac	
 bcap = AIEngine.GetCapacity(ac)
 bdate =  AIEngine.GetDesignDate(ac)
  } else if ((AIEngine.GetCapacity(ac) == bcap) &&  
		   (AIEngine.GetDesignDate(ac) > bdate))	{	// If a tie, we want the newest.
 acid = ac	
 bcap = AIEngine.GetCapacity(ac)
 bdate =  AIEngine.GetDesignDate(ac)
  }
  }
  }
  
if (acid != 0) {
AILog.Info("I've selected " + AIEngine.GetName(acid)  + " to buy.")

// ============================
// build aircraft and give orders
// ============================


local hangar = AIAirport.GetHangarOfAirport(AIBaseStation.GetLocation(ap1));

// AISign.BuildSign(hangar, "VVV");

local newplane = (AIVehicle.BuildVehicle(hangar,acid));

if (AIVehicle.IsValidVehicle(newplane)) {

// set orders

AIOrder.AppendOrder(newplane, AIBaseStation.GetLocation(ap1),(AIOrder.OF_FULL_LOAD_ANY))
AIOrder.AppendOrder(newplane, AIBaseStation.GetLocation(ap2),(AIOrder.OF_FULL_LOAD_ANY))

// name the plane

// ac names

local tn1 = AITown.GetName(AIStation.GetNearestTown(ap1))
local tn2 = AITown.GetName(AIStation.GetNearestTown(ap2))

local acnames = [
"Spirit of " + tn1,
"Flying " + tn1 + "er",
tn1 + " Adventurer",
tn1 + " Rocket",
tn1 + " Star",
tn1 + " Comet",
tn1 + " Prince",
tn1 + " Princess",
"Queen of " + tn1,
"Pride of " + tn1,
"Duke of " + tn1,
tn1 + " Ambassador",
"Wings over " + tn1,
tn1 + " Flash",
tn1 + " Baby",
tn1 + " Wanderer",
"Spirit of " + tn2,
"Flying " + tn2 + "er",
tn2 + " Adventurer",
tn2 + " Rocket",
tn2 + " Star",
tn2 + " Comet",
tn2 + " Prince",
tn2 + " Princess",
"Queen of " + tn2,
"Pride of " + tn2,
"Duke of " + tn2,
tn2 + " Ambassador",
"Wings over " + tn2,
tn2 + " Flash",
tn2 + " Baby",
tn2 + " Wanderer"
]


local nx = 0
local name = null
local changename = false
while (!changename && (nx < 20)) {
nx++ 
name = (acnames[AIBase.RandRange(32)])
changename = AIVehicle.SetName(newplane,name)
}

// start it up

AILog.Info("I've bought a new plane at " + AIBaseStation.GetName(ap1) + " (" + AIVehicle.GetName(newplane) + ").")
AIVehicle.StartStopVehicle(newplane);
} else {
AILog.Info("For some reason (probably not enough money), I couldn't buy an aircraft at " + AIBaseStation.GetName(ap1) + ".")
}



} else {
AILog.Info("I can't find any aircraft available to buy...")
}
} else {
AILog.Info(AIBaseStation.GetName(ap1) + " and " + AIBaseStation.GetName(ap2) + " are too close together for an air route (" + airdist + " tiles).");
}
}
} else {
AILog.Info("I can't find an air route to review."); 
}

// do we want to remove unprofitable and old aircraft?

local rvlist = AIVehicleList();
foreach (veh,z in rvlist) {
if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_AIR) {
rvlist.RemoveItem(veh);
}
}

for (local v = rvlist.Begin(); !(rvlist.IsEnd()); v = rvlist.Next()) {
if  ((AIVehicle.GetProfitLastYear(v) < 0) &&
	(AIVehicle.GetProfitThisYear(v) < 0) &&
	(AIVehicle.GetAge(v) > (365*2))) {
	AIVehicle.SendVehicleToDepot(v);
	AILog.Info(AIVehicle.GetName(v) + " is losing money, so I'm sending it to the hangar.")	
	}
if  (AIVehicle.GetAgeLeft(v) < (365 * 5)) {
	AIVehicle.SendVehicleToDepot(v);
	AILog.Info(AIVehicle.GetName(v) + " is getting old, so I'm sending it to the hangar.")	
	}	
}


}

function CivilAI::AirAssign(station) {

// we just built this airport - reassign aircraft from other airports to fly here.

local aplist = AIStationList(AIStation.STATION_AIRPORT);
local aclist = AIList();
aplist.RemoveItem(station);
aplist.Valuate(AIBase.RandItem); // shuffle the list
local c = 1;
local airdist;
local plane;

// find airports

foreach (airport,z in aplist) {
airdist = AIMap.DistanceManhattan(AIBaseStation.GetLocation(station),AIBaseStation.GetLocation(airport));
if (airdist > MinAirRange) {

// find aircraft
	aclist = AIVehicleList_Station(airport);
			foreach (veh,z in aclist) {
			if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_AIR) {
			aclist.RemoveItem(veh);
			}	
		}

// send a random aircraft to the new airport			
			
			if (aclist.Count() > 1) { // don't send if we have fewer than 2 aircraft serving this airport
				aclist.Valuate(AIBase.RandItem); // shuffle the list
				plane = aclist.Begin(); 
				
				AIOrder.RemoveOrder(plane, 0);
				AIOrder.RemoveOrder(plane, 0);	
				
				AIOrder.AppendOrder(plane, AIBaseStation.GetLocation(airport),(AIOrder.OF_FULL_LOAD_ANY));			
				AIOrder.AppendOrder(plane, AIBaseStation.GetLocation(station),(AIOrder.OF_FULL_LOAD_ANY));
				AILog.Info(AIVehicle.GetName(plane) + " has been redirected to the new airport.")	
			c++;
			}

}
if (c > 3) { break }; // break out of the foreach if we have enough aircraft
}



}

//<
// ====================================================== 
// ====================================================== 
// 			  TTTTT RRRR   U   U    CCC   K   K
// 			    T   R   R  U   U   C   C  K  K
// 			    T   R  R   U   U  C       KK
//  		    T   RRRR   U   U   C   C  K  K
//  		    T   R   R   UUU     CCC   K   K
// ====================================================== 
// ====================================================== 
//>
// ====================================================== 
//                   DELIVER LOCAL GOODS
// ====================================================== 

function CivilAI::TruckOps() {

// depot old trucks

local trucklist = AIList();
local rvlist = AIVehicleList();

foreach (v,z in rvlist) {
if (AIVehicle.GetVehicleType(v) != AIVehicle.VT_ROAD) {
rvlist.RemoveItem(v);
}
local c;
c = FindCargo("MAIL");
if (c > 0) {
if (AIVehicle.GetCapacity(v, c) > 0) {
	 trucklist.AddItem(v, 0);
	 }
}
c = FindCargo("FOOD");
if (c > 0) {
if (AIVehicle.GetCapacity(v, c) > 0) {
	 trucklist.AddItem(v, 0);
	 }
}
c = FindCargo("GOOD");
if (c > 0) {
if (AIVehicle.GetCapacity(v, c) > 0) {
	 trucklist.AddItem(v, 0);
	 }
}
c = FindCargo("VALU");
if (c > 0) {
if (AIVehicle.GetCapacity(v, c) > 0) {
	 trucklist.AddItem(v, 0);
	 }
}
c = FindCargo("WATR");
if (c > 0) {
if (AIVehicle.GetCapacity(v, c) > 0) {
	 trucklist.AddItem(v, 0);
	 }
}
}

for (local v = trucklist.Begin(); !(trucklist.IsEnd()); v = trucklist.Next()) {
if  ((AIVehicle.GetAgeLeft(v) < (365 * 1)) && !(AIVehicle.IsStoppedInDepot(v)) && AIVehicle.IsValidVehicle(v)) {
AIVehicle.SendVehicleToDepot(v);
AILog.Info(AIVehicle.GetName(v) + " is getting old, so I'm sending it to the depot.")
}
}

// new mail stops:

local rv;
if (rv = IdentifyMailTruck(true)) {

 

 // Find a mailcenter to build
 AILog.Info("Looking for a city truck stop to build.");
 
local bslist = AIStationList(AIStation.STATION_BUS_STOP); 

bslist.Valuate(AIStation.GetCargoWaiting, 0); //order by pax waiting
bslist.RemoveBelowValue(150);				
foreach (station, z in bslist) {
if (AIStation.HasStationType(station,AIStation.STATION_TRUCK_STOP)) { // skip existing truck stops
bslist.RemoveItem(station); 
}
}

if (bslist.Count() < 1) {
AILog.Info("I couldn't find one.");
} else {
bslist.Valuate(AIBase.RandItem); // shuffle the list
local newsite = bslist.Begin();
AILog.Info("attempting to build a truck stop at " + AIBaseStation.GetName(newsite) + "."); 

if (BuildTruckStop(newsite,AIBaseStation.GetLocation(newsite))) { // try to build a truck stop

NewMailGo(newsite);

} else {
AILog.Info("I failed to build a truck stop.");
}
}

// build some new trucks for existing routes:
AILog.Info("I'm reviewing my mail services.");


local tslist = AIStationList(AIStation.STATION_TRUCK_STOP); 
tslist.Valuate(AIStation.GetCargoWaiting, FindCargo("MAIL")); //order by mail waiting
tslist.RemoveBelowValue(200);				
foreach (station, z in tslist) {

local vlist = AIVehicleList_Station(station);
foreach (veh,z in vlist) {
if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_ROAD) {
vlist.RemoveItem(veh);
}
}
if (vlist.Count() > 30) {					// remove saturated stops
//AILog.Info(AIBaseStation.GetName(stop) + " has too many buses already.");
} else {
NewMailGo(station);
}
}

AILog.Info("I'm reviewing my cargo services.");

local c = FindCargo("GOOD");
if (c > 0) {
local tslist = AIStationList(AIStation.STATION_TRUCK_STOP); 
tslist.Valuate(AIStation.GetCargoWaiting, c); //order by cargo waiting
tslist.RemoveBelowValue(300);				
foreach (station, z in tslist) {
NewCargoGo(station, c);
}
}

local c = FindCargo("FOOD");
if (c > 0) {
local tslist = AIStationList(AIStation.STATION_TRUCK_STOP); 
tslist.Valuate(AIStation.GetCargoWaiting, c); //order by cargo waiting
tslist.RemoveBelowValue(300);				
foreach (station, z in tslist) {
NewCargoGo(station, c);
}
}

local c = FindCargo("WATR");
if (c > 0) {
local tslist = AIStationList(AIStation.STATION_TRUCK_STOP); 
tslist.Valuate(AIStation.GetCargoWaiting, c); //order by cargo waiting
tslist.RemoveBelowValue(300);				
foreach (station, z in tslist) {
NewCargoGo(station, c);
}
}


return;

}
}

// ====================================================== 
//                   SPIRAL TRUCK STOP
// ====================================================== 
function CivilAI::BuildTruckStop(station,location) {

// clockwise spiral out
local town = AIStation.GetNearestTown(station);
local trytilegrid = [AIMap.GetTileX(location),AIMap.GetTileY(location)]
// AILog.Info(trytilegrid[0] + ", " + trytilegrid[1]); 
local trytile;
local aimx; // aimtile
local aimy; // aimtile
local aimxm; // an additional aimtile, for bridge approach tests (we don't build on bridge approaches, because they send us on journeys)
local aimym; // an additional aimtile, for bridge approach tests (we don't build on bridge approaches, because they send us on journeys)
local testroad = RoadPF();
testroad.cost.no_existing_road = testroad.cost.max_cost;
local x = 0
local y = 0
local i = 0
local stat = station
local statcount = 0
local spread = 4 // footprint size
local count = 1 // number of stops to build

while ((i < spread) && (statcount < count)) {
//y--
for (;y >= 0-i;y--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
aimx  = AIMap.GetTileIndex(trytilegrid[0]+x+1,trytilegrid[1]+y);
aimy  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y+1);
aimxm  = AIMap.GetTileIndex(trytilegrid[0]+x-1,trytilegrid[1]+y);
aimym  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y-1);

//AISign.BuildSign(trytile, "?");

testroad.InitializePath([trytile], [AITown.GetLocation(town)]);
local path = false;
while (path == false) {
path = testroad.FindPath(20);
AIController.Sleep(1);
}

if ((path != null) && (statcount < count) && (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD))) {
	if (!AIBridge.IsBridgeTile(aimx) && !AIBridge.IsBridgeTile(aimxm) && 
	(AITile.GetSlope(aimx) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimxm) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimx,AIRoad.ROADVEHTYPE_TRUCK,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		}
	else if (!AIBridge.IsBridgeTile(aimy) && !AIBridge.IsBridgeTile(aimym) && 
	(AITile.GetSlope(aimy) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimym) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimy,AIRoad.ROADVEHTYPE_TRUCK,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		}
}
}

//x--;
for (;x >= 0-i;x--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
aimx  = AIMap.GetTileIndex(trytilegrid[0]+x+1,trytilegrid[1]+y);
aimy  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y+1);
aimxm  = AIMap.GetTileIndex(trytilegrid[0]+x-1,trytilegrid[1]+y);
aimym  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y-1);

//AISign.BuildSign(trytile, "?");

testroad.InitializePath([trytile], [AITown.GetLocation(town)]);
local path = false;
while (path == false) {
path = testroad.FindPath(20);
AIController.Sleep(1);
}
if ((path != null) && (statcount < count) && (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD))) {
	if (!AIBridge.IsBridgeTile(aimx) && !AIBridge.IsBridgeTile(aimxm) && 
	(AITile.GetSlope(aimx) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimxm) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimx,AIRoad.ROADVEHTYPE_TRUCK,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		}
	else if (!AIBridge.IsBridgeTile(aimy) && !AIBridge.IsBridgeTile(aimym) && 
	(AITile.GetSlope(aimy) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimym) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimy,AIRoad.ROADVEHTYPE_TRUCK,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		}
}
}

//y++;
for (;y <= i;y++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
aimx  = AIMap.GetTileIndex(trytilegrid[0]+x+1,trytilegrid[1]+y);
aimy  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y+1);
aimxm  = AIMap.GetTileIndex(trytilegrid[0]+x-1,trytilegrid[1]+y);
aimym  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y-1);

//AISign.BuildSign(trytile, "?");

testroad.InitializePath([trytile], [AITown.GetLocation(town)]);
local path = false;
while (path == false) {
path = testroad.FindPath(20);
AIController.Sleep(1);
}
if ((path != null) && (statcount < count) && (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD))) {
	if (!AIBridge.IsBridgeTile(aimx) && !AIBridge.IsBridgeTile(aimxm) && 
	(AITile.GetSlope(aimx) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimxm) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimx,AIRoad.ROADVEHTYPE_TRUCK,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		}
	else if (!AIBridge.IsBridgeTile(aimy) && !AIBridge.IsBridgeTile(aimym) && 
	(AITile.GetSlope(aimy) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimym) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimy,AIRoad.ROADVEHTYPE_TRUCK,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		}
}
}

//x++;
for (;x <= i+1;x++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
aimx  = AIMap.GetTileIndex(trytilegrid[0]+x+1,trytilegrid[1]+y);
aimy  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y+1);
aimxm  = AIMap.GetTileIndex(trytilegrid[0]+x-1,trytilegrid[1]+y);
aimym  = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y-1);

//AISign.BuildSign(trytile, "?");

testroad.InitializePath([trytile], [AITown.GetLocation(town)]);
local path = false;
while (path == false) {
path = testroad.FindPath(20);
AIController.Sleep(1);
}
if ((path != null) && (statcount < count) && (AIRoad.HasRoadType(trytile,AIRoad.ROADTYPE_ROAD))) {
	if (!AIBridge.IsBridgeTile(aimx) && !AIBridge.IsBridgeTile(aimxm) && 
	(AITile.GetSlope(aimx) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimxm) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimx,AIRoad.ROADVEHTYPE_TRUCK,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		}
	else if (!AIBridge.IsBridgeTile(aimy) && !AIBridge.IsBridgeTile(aimym) && 
	(AITile.GetSlope(aimy) == AITile.SLOPE_FLAT) &&
	(AITile.GetSlope(aimym) == AITile.SLOPE_FLAT) &&
	AIRoad.BuildDriveThroughRoadStation(trytile,aimy,AIRoad.ROADVEHTYPE_TRUCK,stat)) {
	stat = AIStation.GetStationID(trytile), statcount = statcount+1
		}
}
}
i=i+1
}
if (statcount > 0) { return stat+1 } else { return false }
}




// ====================================================== 
//                    FIND TRUCKS TO BUY
// ======================================================  

function CivilAI::IdentifyMailTruck(silent) {
local busid= 0;
local vlist= 0;
local blist= 0;
local bcap = 0;
local bdate= 0;
local z = 0;

vlist = AIEngineList(AIVehicle.VT_ROAD); 	// list of all road vehicles

foreach (rv,z in vlist) {

//AILog.Info("Assessing " + AIEngine.GetName(rv));

local mail = FindCargo("MAIL");
 
 if (mail == 0) {
  AILog.Info("Mail doesn't seem to exist on this map. Weird.");
return false;
 
 } else {
//  AILog.Info("Mail identified as cargo " + mail);
  }

if ((AIEngine.IsBuildable(rv)) 
&& (AIEngine.CanRefitCargo(rv, mail))					// can we refit mail
&& (AIEngine.GetCapacity(rv) > 9)	
&& (AIEngine.GetRoadType(rv) == AIRoad.ROADTYPE_ROAD)) {

if (AIEngine.GetMaxSpeed(rv) > bcap) {				    // highest speed.
// AILog.Info("!");
 busid = rv+1	
 bcap = AIEngine.GetMaxSpeed(rv)
 bdate =  AIEngine.GetDesignDate(rv)
  } else if ((AIEngine.GetCapacity(rv) == bcap) &&  
		   (AIEngine.GetDesignDate(rv) > bdate))	{	// If a tie, we want the newest.
 busid = rv+1	
 bcap = AIEngine.GetMaxSpeed(rv)
 bdate =  AIEngine.GetDesignDate(rv)
  }
  }
  }
  
if (busid > 0) {
if (!silent) AILog.Info("I've selected " + AIEngine.GetName(busid-1)  + " for mail service.")
return (busid);
} else {
if (!silent) AILog.Info("I can't find any mail trucks available to buy...")
return false;
}
}
// ================
function CivilAI::IdentifyGoodsTruck(silent, cargo) {
local busid= 0;
local vlist= 0;
local blist= 0;
local bcap = 0;
local bgo= 0;
local z = 0;


vlist = AIEngineList(AIVehicle.VT_ROAD); 	// list of all road vehicles

  // prefer native vehicles
 foreach (rv, z in vlist) {
 if ((AIEngine.GetCargoType(rv) != cargo)
 || (AIEngine.GetRoadType(rv) != AIRoad.ROADTYPE_ROAD)) {
 vlist.RemoveItem(rv);
 }
 } 
 if (vlist.Count() > 0) {
 foreach (rv,z in vlist) {
if (AIEngine.GetCapacity(rv) > bcap) {				    // highest capacity.
// AILog.Info("!");
 busid = rv+1	
 bcap = AIEngine.GetCapacity(rv)
 bgo =  AIEngine.GetMaxSpeed(rv)				   
  } else if ((AIEngine.GetCapacity(rv) == bcap) &&  
		   (AIEngine.GetMaxSpeed(rv) > bgo))	{	// If a tie, we want the fastest.
 busid = rv+1	
 bcap = AIEngine.GetCapacity(rv)
 bgo =  AIEngine.GetMaxSpeed(rv) 
 }
 }
 } else {

vlist = AIEngineList(AIVehicle.VT_ROAD);
foreach (rv,z in vlist) {

if ((AIEngine.IsBuildable(rv)) 
&& (AIEngine.CanRefitCargo(rv, cargo))					// can we refit goods
&& (AIEngine.GetCargoType(rv) != FindCargo("MAIL"))		// ignore stock mail trucks (to avoid the courier truck trap!)
&& (AIEngine.GetCapacity(rv) > 9)	
&& (AIEngine.GetRoadType(rv) == AIRoad.ROADTYPE_ROAD)) {

if (AIEngine.GetCapacity(rv) > bcap) {				    // highest capacity.
// AILog.Info("!");
 busid = rv+1	
 bcap = AIEngine.GetCapacity(rv)
 bgo =  AIEngine.GetMaxSpeed(rv)				   
  } else if ((AIEngine.GetCapacity(rv) == bcap) &&  
		   (AIEngine.GetMaxSpeed(rv) > bgo))	{	// If a tie, we want the fastest.
 busid = rv+1	
 bcap = AIEngine.GetCapacity(rv)
 bgo =  AIEngine.GetMaxSpeed(rv)
  }
  }
  }
  }
  
if (busid > 0) {
if (!silent) AILog.Info("I've selected " + AIEngine.GetName(busid-1)  + " for cargo service.")
return (busid);
} else {
if (!silent) AILog.Info("I can't find any cargo trucks available to buy...")
return false;
}
}

// ====================================================== 
//                    MAIL SERVICE
// ======================================================  

function CivilAI::NewMailGo(station) {

local dosh = AICompany.GetBankBalance(Me);
local nbus = IdentifyMailTruck(true);

if (nbus == false) {
return;
}
else if (dosh > (AIEngine.GetPrice(nbus-1) * 2)) {



// check we don't have MaxBus
local rvlist = AIVehicleList();
foreach (veh,z in rvlist) {
if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_ROAD) {
rvlist.RemoveItem(veh);
}
}
rvlist.Valuate(AIVehicle.GetCapacity, FindCargo("MAIL"));
rvlist.KeepAboveValue(9);
local buscount = rvlist.Count();
if (buscount < MaxBus) {

local HasBus = IdentifyMailTruck(true);
if (!HasBus) {
return
}
//AILog.Info("Starting a new mail service.");

local bslist = AIStationList(AIStation.STATION_TRUCK_STOP); 


bslist.RemoveItem(station); // remove ourselves

foreach (stop, z in bslist) {							// remove non-city stops
if (!AIStation.HasStationType(stop,AIStation.STATION_BUS_STOP)) {
bslist.RemoveItem(stop);
}
}

foreach (stop,z in bslist) {
local vlist = AIVehicleList_Station(stop);
foreach (veh,z in vlist) {
if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_ROAD) {
vlist.RemoveItem(veh);
}
}
if (vlist.Count() > 30) {					// remove saturated stops
//AILog.Info(AIBaseStation.GetName(stop) + " has too many buses already.");
bslist.RemoveItem(stop);
}
}

if (bslist.Count() < 1) {
//AILog.Info("I couldn't find a truck stop to connect.");
return
} else {
bslist.Valuate(AIStation.GetCargoWaiting, FindCargo("MAIL"));
local stat2 = bslist.Begin();
CreateRoute(HasBus,2,[station+1,stat2+1],HomeDepot,(AITown.GetName(AIStation.GetNearestTown(station))) + " Mail",false,FindCargo("MAIL"),2);
}

} else {
AILog.Info("I've already got my maximum number of trucks, so I won't build more.")
}
} else {
AILog.Info("I don't have the money for new trucks at the moment.")
}


return
}

// ====================================================== 
//                    CARGO SERVICE
// ====================================================== 


function CivilAI::NewCargoGo(station, cargo) {

local dosh = AICompany.GetBankBalance(Me);
local nbus = IdentifyGoodsTruck(true, cargo);

if (nbus == false) {
return;
}
else if (dosh > (AIEngine.GetPrice(nbus-1) * 2)) {



// check we don't have MaxBus
local rvlist = AIVehicleList();
foreach (veh,z in rvlist) {
if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_ROAD) {
rvlist.RemoveItem(veh);
}
}
rvlist.Valuate(AIVehicle.GetCapacity, cargo);
rvlist.KeepAboveValue(9);
local buscount = rvlist.Count();
if (buscount < MaxBus) {

local HasBus = IdentifyGoodsTruck(true, cargo);
if (!HasBus) {
return
}
//AILog.Info("Adding more goods trucks.");

local cdlist = AIStationList_CargoWaitingByVia(station, cargo);
foreach (d, z in cdlist) {
//AILog.Info(station + " " + d +  " " + z);

if (d != 65535 && z > 200) { // add another truck
CreateRoute(HasBus,1,[station+1,d+1],HomeDepot,(AITown.GetName(AIStation.GetNearestTown(d))) + " Goods",true,cargo,3);
} else if (d == 65535 && z > 200) { // add another truck to a random route

local vlist = AIVehicleList_Station(station);

vlist.Valuate(AIBase.RandItem); // shuffle the list
local blist = AIStationList_Vehicle(vlist.Begin());
blist.RemoveItem(station); // take out this station
local b = blist.Begin(); // set next station as destination

CreateRoute(HasBus,1,[station+1,b+1],HomeDepot,(AITown.GetName(AIStation.GetNearestTown(b))) + " Goods",true,cargo,3);
}



}
} else {
AILog.Info("I've already got my maximum number of trucks, so I won't build more.")
}
} else {
AILog.Info("I don't have the money for new trucks at the moment.")
}


return
}


//=======================================================
// Cargo label identification
//=======================================================


function CivilAI::FindCargo(label) {

local clist = AICargoList();
local y = 0;


 foreach (cargo, y in clist) {
// AILog.Info(AICargo.GetCargoLabel(cargo));
 if (AICargo.GetCargoLabel(cargo) == label) {
// AILog.Info("Bingo.");
 return cargo;
 } 
 }
return;
 }

 
//=======================================================
// Cargo route planning
//=======================================================


function CivilAI::CargoPlan() {

AILog.Info("I'm coming up with a cargo plan...");

local food = FindCargo("FOOD");
if (food > 0) {
MakeAPlan(food);
} else {
//AILog.Info("There is no food in this game.");
}

local watr = FindCargo("WATR");
if (watr > 0) {
MakeAPlan(watr);
} else {
//AILog.Info("There is no water in this game.");
}


local good = FindCargo("GOOD");
if (good > 0) {
MakeAPlan(good);
} else {
//AILog.Info("There are no goods in this game (which is odd...)");
}

}

function CivilAI::MakeAPlan(cargo) {

//=== First, find the destination

local tslist = AIStationList(AIStation.STATION_TRUCK_STOP); 

foreach (stop, z in tslist) {
if ((AITile.GetCargoAcceptance(AIBaseStation.GetLocation(stop), cargo, 1, 1, 3) > 7) 
&& (AITown.GetLastMonthReceived(AIStation.GetNearestTown(stop), AICargo.GetTownEffect(cargo)) == 0)
&& AIStation.HasStationType(stop,AIStation.STATION_BUS_STOP)) { // only consider stations with bus stops, ie not supply depots
	//this is a valid destination
} else {
tslist.RemoveItem(stop);
}
}
if (tslist.Count() > 0) {
tslist.Valuate(AIBase.RandItem); // shuffle the list
PlanDestination = tslist.Begin();
AILog.Info("Planning a cargo run to " + AIBaseStation.GetName(PlanDestination) + ".");

//=== Then, find the supplier

//=== do we happen to have a nearby existing supply?

local tslist = AIStationList(AIStation.STATION_TRUCK_STOP); 
foreach (stop, z in tslist) {
if ((AIStation.GetCargoWaiting(stop, cargo) > 64) 
&& (AIMap.DistanceManhattan(AIBaseStation.GetLocation(stop),AIBaseStation.GetLocation(PlanDestination)) < TrainRange)
&& (stop != PlanDestination)) // don't try and deliver to ourselves
{


AILog.Info("Existing supply depot found at " + AIBaseStation.GetName(stop) + ".");

// build service

			local a = [AIBaseStation.GetLocation(stop)]
			local b	= [AIBaseStation.GetLocation(PlanDestination)]				
			BuildARoad(a,b,-1); // just in case

local HasBus = IdentifyGoodsTruck(false, cargo);
if (!HasBus) { return; } else {
CreateRoute(HasBus,2,[stop+1,PlanDestination+1],HomeDepot,(AITown.GetName(AIStation.GetNearestTown(PlanDestination))) + " Goods",true,cargo,3);
}

return;
}
}

//AILog.Info("No existing supply depot found.");
//=== if not, is there an undertransported industry nearby?

local ilist = AIIndustryList();

foreach (i, z in ilist) {
local prod = false;
local clist = AIIndustryType.GetProducedCargo(AIIndustry.GetIndustryType(i));
if (clist.Count() > 0) {
foreach (c, z in clist) {
if (c == cargo) {
prod = true;
}
}
}
if (prod == false) {
ilist.RemoveItem(i);
} else if (AIIndustry.IsBuiltOnWater(i)) {
ilist.RemoveItem(i);
} else if (AIMap.DistanceManhattan((AIIndustry.GetLocation(i)),AIBaseStation.GetLocation(PlanDestination)) > TrainRange * 2) {
ilist.RemoveItem(i);
} else if (AIIndustry.GetLastMonthProduction(i, cargo) - AIIndustry.GetLastMonthTransported(i, cargo) < 64) {
ilist.RemoveItem(i);
}
}
if (ilist.Count() > 0) {
ilist.Valuate(AIBase.RandItem); // shuffle the list
PlanSource = ilist.Begin();
AILog.Info("Suitable supplier found at " + AIIndustry.GetName(PlanSource) + ".");

// do we already have a supply depot at this industry?

local search = AITileList_IndustryProducing(PlanSource, 3);
foreach (tile, z in search) {
//AISign.BuildSign(tile, ".");

if (AITile.IsStationTile(tile)) {
local station = AIStation.GetStationID(tile);

if (AIStation.IsValidStation(station)
&&  AIStation.HasStationType(station,AIStation.STATION_TRUCK_STOP)
&& !(AIStation.HasStationType(station,AIStation.STATION_BUS_STOP))
) {
// this is a valid supply depot
local stop = station;
AILog.Info("Existing supply depot found at " + AIBaseStation.GetName(stop) + ".");

// build service

			local a = [AIBaseStation.GetLocation(stop)]
			local b	= [AIBaseStation.GetLocation(PlanDestination)]				
			BuildARoad(a,b,-1); // just in case

local HasBus = IdentifyGoodsTruck(false, cargo);
if (!HasBus) { return; } else {

// don't be lazy; find the supply depot depot
//AILog.Info("Looking for depot.");
local tt; 
local depot = HomeDepot;

for(local x = -3; x < 6; x++) {
for(local y = -3; y < 6; y++) {

tt = AIMap.GetTileIndex (AIMap.GetTileX(AIBaseStation.GetLocation(stop)) + x, AIMap.GetTileY(AIBaseStation.GetLocation(stop)) + y);
//AISign.BuildSign(tt, "?");
 if (AIRoad.IsRoadDepotTile(tt)) {
  depot = tt
  break;
 }
 }
 if (depot != HomeDepot) {
// AILog.Info("Found depot.");
 break;
 }
 }

CreateRoute(HasBus,2,[stop+1,PlanDestination+1],depot,(AITown.GetName(AIStation.GetNearestTown(PlanDestination))) + " Goods",true,cargo,3);
}

return;
}
}
}


// build supply depot and service
local stop;
if (stop = SupplyDepot(PlanSource)) {		

			local a = [AIBaseStation.GetLocation(stop)]
			local b	= [AIBaseStation.GetLocation(PlanDestination)]				
			BuildARoad(a,b,-1);

local HasBus = IdentifyGoodsTruck(false, cargo);
if (!HasBus) { return; } else {

// don't be lazy; find the supply depot depot
//AILog.Info("Looking for depot.");
local tt; 
local depot = HomeDepot;

for(local x = -3; x < 6; x++) {
for(local y = -3; y < 6; y++) {

tt = AIMap.GetTileIndex (AIMap.GetTileX(AIBaseStation.GetLocation(stop)) + x, AIMap.GetTileY(AIBaseStation.GetLocation(stop)) + y);
//AISign.BuildSign(tt, "?");
 if (AIRoad.IsRoadDepotTile(tt)) {
  depot = tt
  break;
 }
 }
 if (depot != HomeDepot) {
// AILog.Info("Found depot.");
 break;
 }
 }

CreateRoute(HasBus,2,[stop+1,PlanDestination+1],depot,(AITown.GetName(AIStation.GetNearestTown(PlanDestination))) + " Goods",true,cargo,3);
}
}
return;
}
AILog.Info("I couldn't find a suitable supply.");

} else {
AILog.Info("I couldn't find a suitable destination.");
return;
}
}

//===============
// Build a supply depot
//===============

function CivilAI::SupplyDepot(ind) {



local dosh = AICompany.GetBankBalance(Me);
if ((dosh) < (AIRoad.GetBuildCost(AIRoad.ROADTYPE_ROAD, AIRoad.BT_ROAD) * 100)) {
AILog.Info("I can't afford to build a supply depot right now. Perhaps later.")

return null;
}	


// clockwise spiral out
local trytilegrid = [AIMap.GetTileX(AIIndustry.GetLocation(ind)) + 1,AIMap.GetTileY(AIIndustry.GetLocation(ind)) + 1] // we're making assumptions about industry size here
local trytile;
local x = 0
local y = 0
local i = 0
local s = 4
local NewStat = null;


while ((i < s) && NewStat == null) {
//y--
for (;y >= 0-i;y--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if (AITile.IsBuildableRectangle(trytile, 3, 3)) {
if (NewStat = BuildSupplyDepot(trytile,AIIndustry.GetLocation(ind))) {
return NewStat;
}
}
} 
//x--;
for (;x >= 0-i;x--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if (AITile.IsBuildableRectangle(trytile, 3, 3)) {
if (NewStat = BuildSupplyDepot(trytile,AIIndustry.GetLocation(ind))) {
return NewStat;
}
}
} 
//y++;
for (;y <= i;y++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if (AITile.IsBuildableRectangle(trytile, 3, 3)) {
if (NewStat = BuildSupplyDepot(trytile,AIIndustry.GetLocation(ind))) {
return NewStat;
}
}
} 
//x++;
for (;x <= i;x++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if (AITile.IsBuildableRectangle(trytile, 3, 3)) {
if (NewStat = BuildSupplyDepot(trytile,AIIndustry.GetLocation(ind))) {
return NewStat;
}
}
} 
i=i+1
}

return NewStat;

}

function CivilAI::BuildSupplyDepot(tile,indloc) {

// check town approval before levelling mountains!
//AILog.Info(AITown.GetRating(AITile.GetClosestTown(tile), Me))
if (AITown.GetRating(AITile.GetClosestTown(tile), Me) < 3 && AITown.GetRating(AITile.GetClosestTown(tile), Me) > 0) {
return null;
}


local tg = [AIMap.GetTileX(tile),AIMap.GetTileY(tile)]

// flatten land

AITile.LevelTiles(indloc,AIMap.GetTileIndex(tg[0],tg[1]));
AITile.LevelTiles(indloc,AIMap.GetTileIndex(tg[0],tg[1]));
AITile.LevelTiles(indloc,AIMap.GetTileIndex(tg[0],tg[1]+3));
AITile.LevelTiles(indloc,AIMap.GetTileIndex(tg[0]+3,tg[1]));
AITile.LevelTiles(indloc,AIMap.GetTileIndex(tg[0]+3,tg[1]+3));


// cont.

AIRoad.SetCurrentRoadType(AIRoad.ROADTYPE_ROAD);

AIRoad.BuildRoad (AIMap.GetTileIndex(tg[0],tg[1]    ),AIMap.GetTileIndex(tg[0],tg[1] +2));
AIRoad.BuildRoad (AIMap.GetTileIndex(tg[0],tg[1]    ),AIMap.GetTileIndex(tg[0] +2,tg[1]));
AIRoad.BuildRoad (AIMap.GetTileIndex(tg[0],tg[1] +2),AIMap.GetTileIndex(tg[0] +2,tg[1] +2));
AIRoad.BuildRoad (AIMap.GetTileIndex(tg[0] +2,tg[1]),AIMap.GetTileIndex(tg[0] +2,tg[1] +2));

local stat = AIStation.STATION_NEW

if (AIBase.RandRange(2) == 1) {

AIRoad.BuildDriveThroughRoadStation(AIMap.GetTileIndex(tg[0]+1,tg[1]),AIMap.GetTileIndex(tg[0]+2,tg[1]),AIRoad.ROADVEHTYPE_TRUCK,stat);
stat = AIStation.GetStationID(AIMap.GetTileIndex(tg[0]+1,tg[1]));
AIRoad.BuildDriveThroughRoadStation(AIMap.GetTileIndex(tg[0]+1,tg[1]+2),AIMap.GetTileIndex(tg[0]+2,tg[1]+2),AIRoad.ROADVEHTYPE_TRUCK,stat);
AIRoad.BuildRoadDepot(AIMap.GetTileIndex(tg[0]+1,tg[1]+1),AIMap.GetTileIndex(tg[0]+2,tg[1]+1));
AIRoad.BuildRoad (AIMap.GetTileIndex(tg[0]+1,tg[1]+1),AIMap.GetTileIndex(tg[0]+2,tg[1]+1));

} else {

AIRoad.BuildDriveThroughRoadStation(AIMap.GetTileIndex(tg[0],tg[1]+1),AIMap.GetTileIndex(tg[0],tg[1]+2),AIRoad.ROADVEHTYPE_TRUCK,stat);
stat = AIStation.GetStationID(AIMap.GetTileIndex(tg[0],tg[1]+1));
AIRoad.BuildDriveThroughRoadStation(AIMap.GetTileIndex(tg[0]+2,tg[1]+1),AIMap.GetTileIndex(tg[0]+2,tg[1]+2),AIRoad.ROADVEHTYPE_TRUCK,stat);
AIRoad.BuildRoadDepot(AIMap.GetTileIndex(tg[0]+1,tg[1]+1),AIMap.GetTileIndex(tg[0]+1,tg[1]+2));
AIRoad.BuildRoad (AIMap.GetTileIndex(tg[0]+1,tg[1]+1),AIMap.GetTileIndex(tg[0]+1,tg[1]+2));

}

AILog.Info("Built a Supply Depot.")
return stat;
}



// ====================================================== 
// ====================================================== 
// 			  TTTTT RRRR     A    I  N   N
// 			    T   R   R   A A   I  NN  N
// 			    T   R  R   A   A  I  N N N
//  		    T   RRRR   AAAAA  I  N  NN
//  		    T   R   R  A   A  I  N   N
// ===========================================ChooChoo!== 
// ====================================================== 

function CivilAI::ChooChoo() {

if (AIGameSettings.IsDisabledVehicleType(AIVehicle.VT_RAIL)) {
return; // no trains for us
}


AILog.Info("I'm thinking about trains...");
// =================================================================
// review existing trains
// =================================================================


local trainlist = AIVehicleList();
foreach (v, z in trainlist) {
if (AIVehicle.GetVehicleType(v) != AIVehicle.VT_RAIL) {
trainlist.RemoveItem(v);
}
}

// trains getting old
foreach (v, z in trainlist) {
if  ((AIVehicle.GetAgeLeft(v) < (365 * 1)) && !(AIVehicle.IsStoppedInDepot(v)) && AIVehicle.IsValidVehicle(v)) {

AILog.Info(AIVehicle.GetName(v) + " is getting old")

// replace the train before sending it to the depot
local dosh = AICompany.GetBankBalance(Me);
local dest = AIStation.GetStationID(AIOrder.GetOrderDestination(v, 1));
local src = AIStation.GetStationID(AIOrder.GetOrderDestination(v, 0));
local cargo = 0;
local clist = AICargoList();
foreach (c, z in clist) {
if (AIVehicle.GetCapacity(v, c) > 0) {
cargo = c;
}
}
local depot = FindTrainDepot(dest);

if (dosh > BuyATrain(dest, src, cargo, depot, true) && AIRail.IsRailDepotTile(depot)) {

//AILog.Info(AIBaseStation.GetName(dest) + ", " + AIBaseStation.GetName(src)  + ", " + cargo + ", " + AIMap.GetTileX(depot) + ", " + AIMap.GetTileY(depot) )
BuyATrain(dest, src, cargo, depot, false);
AIVehicle.SendVehicleToDepot(v);
AIOrder.AppendOrder(v,depot,(AIOrder.OF_STOP_IN_DEPOT));
AILog.Info("I replaced it.")
} else {
AILog.Info("I can't afford to replace it (or something went wrong).")
}
}
}

// trains losing money
foreach (v, z in trainlist) {

local dest = AIStation.GetStationID(AIOrder.GetOrderDestination(v, 1));
local depot = FindTrainDepot(dest);

if  ((AIVehicle.GetProfitLastYear(v) < 0) &&
	(AIVehicle.GetProfitThisYear(v) < 0) &&
   !(AIVehicle.IsStoppedInDepot(v)) &&
	(AIVehicle.GetAge(v) > (365*2)) &&
	 (AIVehicle.GetAgeLeft(v) > (365 * 1))) {
AIVehicle.SendVehicleToDepot(v);
AIOrder.AppendOrder(v,depot,(AIOrder.OF_STOP_IN_DEPOT));
	AILog.Info(AIVehicle.GetName(v) + " is losing money, so I'm sending it to the depot.")	
	}
}

// =================================================================
// clear up abandoned tracks
// =================================================================

local tslist = AIStationList(AIStation.STATION_TRAIN); 
foreach (ts, z in tslist) {
local tlist = AIVehicleList_Station(ts);
if (tlist.Count() == 0) {

// remove dest, depot and tracks

local depot = FindTrainDepot(ts);
if (AIRail.IsRailDepotTile(depot)) {
local dg = [AIMap.GetTileX(depot),AIMap.GetTileY(depot)]
if (AIRail.GetRailStationDirection(AIBaseStation.GetLocation(ts)) == AIRail.RAILTRACK_NW_SE) {
AITile.DemolishTile(AIMap.GetTileIndex(dg[0] + 1, dg[1] + 1));
AITile.DemolishTile(AIMap.GetTileIndex(dg[0] + 1, dg[1]));
AITile.DemolishTile(AIMap.GetTileIndex(dg[0] + 1, dg[1] - 1));
} else {
AITile.DemolishTile(AIMap.GetTileIndex(dg[0] + 1, dg[1] + 1));
AITile.DemolishTile(AIMap.GetTileIndex(dg[0], dg[1] +1));
AITile.DemolishTile(AIMap.GetTileIndex(dg[0] - 1, dg[1] + 1));
}
AITile.DemolishTile(depot);
AITile.DemolishTile(AIBaseStation.GetLocation(ts));
} else {


// remove supply and track

RemoveTracksFrom(ts);
AITile.DemolishTile(AIBaseStation.GetLocation(ts));
}
}
}

// =================================================================
// BUILD NEW TRAINS :D
// =================================================================

local rvlist = AIVehicleList();
foreach (veh,z in rvlist) {
if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_ROAD) {
rvlist.RemoveItem(veh);
}
}
rvlist.Valuate(AIVehicle.GetCapacity, 0);
rvlist.KeepAboveValue(9);
local buscount = rvlist.Count();

rvlist = AIVehicleList();
foreach (veh,z in rvlist) {
if (AIVehicle.GetVehicleType(veh) != AIVehicle.VT_RAIL) {
rvlist.RemoveItem(veh);
}
}
local traincount = rvlist.Count();
if (traincount > buscount / 9) {
//AILog.Info("...but I have enough already - I'm supposed to be a road AI!");
//return;
}

local c = FindCargo("GOOD");
if (c > 0) {
CargoTerminus(c);
} else {
//AILog.Info("There are no goods in this game (which is odd...)");
}

local f = FindCargo("FOOD");
if (f > 0) {
CargoTerminus(f);
} else {
//AILog.Info("There is no food in this game.)");
}


}


function CivilAI::CargoTerminus(cargo) {

local dest;
local src;
local suptile;
local go = false;
local secline = false;


// find an industry to build a terminus at (or whose terminus we shall reuse)
// first, examine existing termini

local conrail;
local contile;
local intile;
local gotile;
local sigtile;
local sigtar = [0,0,0];

local tslist = AIStationList(AIStation.STATION_TRAIN); 
tslist.RemoveList(DudTerminus); // remove known unreachables

// check for connectability, then for suppliers

foreach (ts, z in tslist) {
local depot = FindTrainDepot(ts);
if (AIRail.IsRailDepotTile(depot)) { // we found a depot, so this is a terminus station
local dg = [AIMap.GetTileX(depot),AIMap.GetTileY(depot)]
local doff = [AIMap.GetTileX(depot) - AIMap.GetTileX(AIBaseStation.GetLocation(ts)),AIMap.GetTileY(depot) - AIMap.GetTileY(AIBaseStation.GetLocation(ts))]
//AILog.Info(doff[0] + ", " + doff[1])

local isclear = false;

	if (doff[0] == -2) { // depot northeast
	
contile = AIMap.GetTileIndex(dg[0]-1,dg[1]+1);
sigtile = AIMap.GetTileIndex(dg[0]-2,dg[1]+1);
sigtar[0] = AIMap.GetTileIndex(dg[0]-3,dg[1]+1);
sigtar[1] = AIMap.GetTileIndex(dg[0]-2,dg[1]+2);
sigtar[2] = AIMap.GetTileIndex(dg[0]-2,dg[1]);
conrail = AIRail.RAILTRACK_SW_SE
intile = AIMap.GetTileIndex(dg[0]-1,dg[1]+2);
gotile = AIMap.GetTileIndex(dg[0]-1,dg[1]+3);

	if (AITile.IsBuildableRectangle(intile, 1, 2) && AITile.IsBuildableRectangle(AIMap.GetTileIndex(dg[0]-2,dg[1]+3), 3, 1)) {
//		AISign.BuildSign(AIMap.GetTileIndex(dg[0]-2,dg[1]+3), "clear")
		isclear = true;
	}
	
	
} else if (doff[1] == -2) { // depot northwest

contile = AIMap.GetTileIndex(dg[0]+1,dg[1]-1);
sigtile = AIMap.GetTileIndex(dg[0]+1,dg[1]-2);
sigtar[0] = AIMap.GetTileIndex(dg[0]+1,dg[1]-3);
sigtar[1] = AIMap.GetTileIndex(dg[0]+2,dg[1]-2);
sigtar[2] = AIMap.GetTileIndex(dg[0],dg[1]-2);
conrail = AIRail.RAILTRACK_SW_SE
intile = AIMap.GetTileIndex(dg[0]+2,dg[1]-1);
gotile = AIMap.GetTileIndex(dg[0]+3,dg[1]-1);

	if (AITile.IsBuildableRectangle(intile, 1, 2) && AITile.IsBuildableRectangle(AIMap.GetTileIndex(dg[0]+3,dg[1]-2), 3, 1)) {
//		AISign.BuildSign(AIMap.GetTileIndex(dg[0]+3,dg[1]-2), "clear")
		isclear = true;
	}


} else if (doff[0] == 4) { // depot southwest

contile = AIMap.GetTileIndex(dg[0]+1,dg[1]+1);
sigtile = AIMap.GetTileIndex(dg[0]+2,dg[1]+1);
sigtar[0] = AIMap.GetTileIndex(dg[0]+3,dg[1]+1);
sigtar[1] = AIMap.GetTileIndex(dg[0]+2,dg[1]+2);
sigtar[2] = AIMap.GetTileIndex(dg[0]+2,dg[1]);
conrail = AIRail.RAILTRACK_NE_SE
intile = AIMap.GetTileIndex(dg[0]+1,dg[1]+2);
gotile = AIMap.GetTileIndex(dg[0]+1,dg[1]+3);

	if (AITile.IsBuildableRectangle(intile, 1, 2) && AITile.IsBuildableRectangle(AIMap.GetTileIndex(dg[0],dg[1]+3), 3, 1)) {
//		AISign.BuildSign(AIMap.GetTileIndex(dg[0],dg[1]+3), "clear")
		isclear = true;
	}


} else { // depot southeast

contile = AIMap.GetTileIndex(dg[0]+1,dg[1]+1);
sigtile = AIMap.GetTileIndex(dg[0]+1,dg[1]+2);
sigtar[0] = AIMap.GetTileIndex(dg[0]+1,dg[1]+3);
sigtar[1] = AIMap.GetTileIndex(dg[0]+2,dg[1]+2);
sigtar[2] = AIMap.GetTileIndex(dg[0],dg[1]+2);
conrail = AIRail.RAILTRACK_NW_SW
intile = AIMap.GetTileIndex(dg[0]+2,dg[1]+1);
gotile = AIMap.GetTileIndex(dg[0]+3,dg[1]+1);

	if (AITile.IsBuildableRectangle(intile, 1, 2) && AITile.IsBuildableRectangle(AIMap.GetTileIndex(dg[0]+3,dg[1]), 3, 1)) {
//		AISign.BuildSign(AIMap.GetTileIndex(dg[0]+3,dg[1]), "clear")
		isclear = true;
	}
}


//AISign.BuildSign(contile, "o")
//AISign.BuildSign(intile, "/")
//AISign.BuildSign(gotile, "^")
//AISign.BuildSign(sigtile, "s")
//AISign.BuildSign(sigtar[0], "s-")


if (isclear) { // the station has access for a second line, let's see if we can find a supplier

local supdist = 10000000;
suptile = null;

local clist = AICargoList_StationAccepting(ts)
foreach (c, z in clist) {
local suplist = AIIndustryList_CargoProducing(c);
suplist.RemoveList(DudIndustries); // remove known unconnectables


// no double-dipping
foreach (s, z in suplist) {
local d = AIMap.DistanceManhattan((contile),(AIIndustry.GetLocation(s)));
if (d < TrainRange
&& AIIndustry.GetLastMonthProduction(s, c) > 60
&& AIIndustry.GetLastMonthTransportedPercentage(s, c) == 0
&& !AIIndustry.IsBuiltOnWater(s)) {
 foreach (t, z in AITileList_IndustryProducing(s, 4)) {
 local stat = AIStation.GetStationID(t);
if (AIStation.IsValidStation (stat)){
 foreach (train, z in AIVehicleList_Station(stat)) {
 if (AIVehicle.GetCapacity 	(train, c)) {
 suplist.RemoveItem(s);
}	
}
}
}
}
}

foreach (s, z in suplist) {
local d = AIMap.DistanceManhattan((contile),(AIIndustry.GetLocation(s)));
//AILog.Info("Assessing " + AIIndustry.GetName(s) + " (" + d + " tiles).");

if (d < TrainRange
&& AIIndustry.GetLastMonthProduction(s, c) > 60
&& AIIndustry.GetLastMonthTransportedPercentage(s, c) == 0
&& !AIIndustry.IsBuiltOnWater(s)) {

if (d < supdist) {
supdist = d; // this is the nearest supplier
suptile = (AIIndustry.GetLocation(s));

// check for industries which don't have a tile in their top left corner (PITA)
local i = 0;
while (!AIIndustry.IsValidIndustry(AIIndustry.GetIndustryID(suptile)) && i < 10) {
suptile = AIMap.GetTileIndex((AIMap.GetTileX(suptile) + 1),(AIMap.GetTileY(suptile) + 1));
//AILog.Info(AIIndustry.GetName(AIIndustry.GetIndustryID(suptile)))
i++
}

cargo = c; //reusing this label, naughty!
}
} else {
suplist.RemoveItem(s);
}
}
}



if (suptile != null) { // we found a supplier, let's do it!
dest = ts;
//AILog.Info("dest: " + dest + " suptile: " + suptile);
break;
}
} else {
DudTerminus.AddItem(ts, 0); // remember terminus as unreachable
}
}
}


if (dest != null && suptile != null) { // We found a suitable terminus, so build a line
AILog.Info("Contemplating " + AIIndustry.GetName(AIIndustry.GetIndustryID(suptile)) + " -> " + AIBaseStation.GetName(dest) + ".");
// check we have dosh

local dosh = AICompany.GetBankBalance(Me);
if (dosh < (
(AIRail.GetBuildCost(0, AIRail.BT_TRACK) * 2 * TrainRange) +
(AITile.GetBuildCost(AITile.BT_CLEAR_FIELDS) * TrainRange) +
(AIRail.GetBuildCost(0, AIRail.BT_STATION) * 3) +
BuyATrain(0, 0, cargo, 0, true)
)) {
AILog.Info("I can't afford to build a railway right now. Perhaps later.")
return;
}
	
	if (src = TrainPickup(suptile, AIIndustry.GetIndustryID(suptile), cargo)) {	
	secline = true;
	go = true;
	} else {
	DudIndustries.AddItem(AIIndustry.GetIndustryID(suptile), 0);
	}

} else {
//=================================
//We need a new terminus. create a list of goods/food producing industries near our network, with no current production.

local BuildList = AIList();
local indlist = AIIndustryList();
local townlist = MyTownList();

indlist.RemoveList(DudIndustries); // remove known unconnectables

foreach (i, z in indlist) {
local clist = AIIndustryType.GetProducedCargo(AIIndustry.GetIndustryType(i));
if (clist != null) {
if (clist.Count() == 0) {
BuildList.AddItem(i,0) // add non-transporters (ie water towers and power stations) to the list of possibles?
} else {
foreach (c, z in clist) {
if (c == cargo) {
BuildList.AddItem(i,0) // add this industry to the list of possibles
}
}
}
}
}

local tslist = AIStationList(AIStation.STATION_TRAIN); 
foreach (i, z in BuildList) {
local catchment = AITileList_IndustryAccepting(i, 4);
foreach (stat, z in tslist) {
if (catchment.HasItem(AIStation.GetLocation(stat))) {
BuildList.RemoveItem(i); // remove industries with existing stations
}
}
}

foreach (i, z in BuildList) {
local tiletown = AITile.GetClosestTown(AIIndustry.GetLocation(i));

if (AIIndustry.GetLastMonthProduction(i, cargo) > 0) { // remove industries which are already supplied
BuildList.RemoveItem(i);
} else if (!(townlist.HasItem(tiletown))) { // remove industries not near our towns
BuildList.RemoveItem(i);
} else if (AIIndustry.IsBuiltOnWater(i)) {
BuildList.RemoveItem(i);
}
}

//Now check for nearby suppliers

foreach (i, z in BuildList) {
//AILog.Info("Contemplating " + AIIndustry.GetName(i) + ".");

// Find the nearest resource supplier
local suplist = AIList();
local cslist = AIIndustryType.GetAcceptedCargo(AIIndustry.GetIndustryType(i));
local supcount = 0;

foreach (c, z in cslist) {
suplist = AIIndustryList_CargoProducing(c);
suplist.RemoveList(DudIndustries); // remove known unconnectables
foreach (s, z in suplist) {
local d = AIMap.DistanceManhattan((AIIndustry.GetLocation(i)),(AIIndustry.GetLocation(s)));
//AILog.Info("Assessing " + AIIndustry.GetName(s) + " (" + d + " tiles).");

if (d < TrainRange
&& AIIndustry.GetLastMonthProduction(s, c) > 60
&& AIIndustry.GetLastMonthTransportedPercentage(s, c) == 0
&& !AIIndustry.IsBuiltOnWater(s)) {
supcount++ // this is an appropriate supplier
} else {
suplist.RemoveItem(s);
}
}
}
if (supcount == 0) {
//AILog.Info("No suppliers found in range.");
BuildList.RemoveItem(i);
} else {
//AILog.Info(supcount + " suppliers found.");
}
}

if (BuildList.Count() > 0) {
BuildList.Valuate(AIBase.RandItem); // shuffle the list
local Ind = BuildList.Begin();

AILog.Info("Contemplating " + AIIndustry.GetName(Ind) + ".");

// now we check again, to get the correct direction for the station
// Find the nearest resource supplier
local suplist = AIList();
local cslist = AIIndustryType.GetAcceptedCargo(AIIndustry.GetIndustryType(Ind));
local supdist = 10000000;
suptile = null;

foreach (c, z in cslist) {
suplist = AIIndustryList_CargoProducing(c);
suplist.RemoveList(DudIndustries); // remove known unconnectables
foreach (s, z in suplist) {
local d = AIMap.DistanceManhattan((AIIndustry.GetLocation(Ind)),(AIIndustry.GetLocation(s)));
//AILog.Info("Assessing " + AIIndustry.GetName(s) + " (" + d + " tiles).");

if (d < TrainRange
&& AIIndustry.GetLastMonthProduction(s, c) > 60
&& AIIndustry.GetLastMonthTransportedPercentage(s, c) == 0
&& !AIIndustry.IsBuiltOnWater(s)) {

if (d < supdist) {
supdist = d; // this is the nearest supplier
suptile = (AIIndustry.GetLocation(s));

// check for industries which don't have a tile in their top left corner (PITA)
local i = 0;
while (!AIIndustry.IsValidIndustry(AIIndustry.GetIndustryID(suptile)) && i < 10) {
suptile = AIMap.GetTileIndex((AIMap.GetTileX(suptile) + 1),(AIMap.GetTileY(suptile) + 1));
//AILog.Info(AIIndustry.GetName(AIIndustry.GetIndustryID(suptile)))
i++
}

cargo = c; //reusing this label, naughty!
}
} else {
suplist.RemoveItem(s);
}
}
}

// didn't find a supplier

if (suptile == null) {
AILog.Info("I failed to find a supplier.")
return;
}



//Now pick where to put the station, and in what orientation

local xdist = AIMap.GetTileX(suptile) - AIMap.GetTileX(AIIndustry.GetLocation(Ind));
local ydist = AIMap.GetTileY(suptile) - AIMap.GetTileY(AIIndustry.GetLocation(Ind));
local xabs;
local yabs;
if (xdist < 0) { xabs = 0 - xdist; } else { xabs = xdist; }
if (ydist < 0) { yabs = 0 - ydist; } else { yabs = ydist; }

AILog.Info("Supplier found at " + AIIndustry.GetName(AIIndustry.GetIndustryID(suptile)) + ".");

local s = 0;
local o = 0;

if (xabs > yabs) { 	// supplier is on the x axis
	if (xdist > 0) {
		if (ydist > 0) {
						s = 0;
						o = 0;
		} else {
						s = 2;
						o = 0;
		}
	} else {	
		if (ydist > 0) {
						s = 0;
						o = 1;
		} else {
						s = 2;
						o = 1;
		}	
	}
} else {			// supplier is on the y axis
	if (ydist > 0) { 
		if (xdist > 0) {
						s = 1;
						o = 0;
		} else {
						s = 3;
						o = 0;
		}
	} else {	
		if (xdist > 0) {
						s = 1;
						o = 1;
		} else {
						s = 3;
						o = 1;
		}	
	}

}

// Let's build the route! (if we have the money)
// check funds allow for the connection, and a bit left over to maintain the network

local dosh = AICompany.GetBankBalance(Me);
if (dosh < (
(AIRail.GetBuildCost(0, AIRail.BT_TRACK) * 2 * TrainRange) +
(AITile.GetBuildCost(AITile.BT_CLEAR_FIELDS) * TrainRange) +
(AIRail.GetBuildCost(0, AIRail.BT_STATION) * 6) +
(AIRail.GetBuildCost(0, AIRail.BT_DEPOT)) +
BuyATrain(0, 0, cargo, 0, true)
)) {
AILog.Info("I can't afford to build a railway right now. Perhaps later.")
return;
}	

if (dest = SmallStation(Ind, s, o, cargo)) {
//AILog.Info("ping " + AIStation.GetName(dest))
	if (src = TrainPickup(suptile, AIIndustry.GetIndustryID(suptile), cargo)) {
	go = true;
	} else {
	DudIndustries.AddItem(AIIndustry.GetIndustryID(suptile), 0);
	}
} else if (dest = SmallStation(Ind, (s + 2) % 4, o, cargo)) {
//AILog.Info("pong " + AIStation.GetName(dest))
	if (src = TrainPickup(suptile, AIIndustry.GetIndustryID(suptile), cargo)) {
	go = true;
	} else {
	DudIndustries.AddItem(AIIndustry.GetIndustryID(suptile), 0);
	}
} else {
DudIndustries.AddItem(Ind, 0);
AILog.Info("I couldn't find a station location at " + AIIndustry.GetName(Ind))
}
} else {
AILog.Info("I couldn't find an appropriate industry to serve.");

DudCounter++

if (DudCounter > 3) {
DudIndustries.Valuate(AIBase.RandItem); // shuffle the list
DudIndustries.RemoveTop(1);// remove a random industry from the dud list
DudCounter = 0;
}

// see if we can't find a nearby industry in a smaller town

local oilist = AIIndustryList();
local xoilist = AIList();

foreach (i, z in oilist) {
local clist = AIIndustryType.GetProducedCargo(AIIndustry.GetIndustryType(i));
if (clist != null) {
if (clist.Count() == 0) {
xoilist.AddItem(i,0) // add non-transporters (ie water towers and power stations) to the list of possibles?
} else {
foreach (c, z in clist) {
if (c == cargo) {
xoilist.AddItem(i,0) // add this industry to the list of possibles
}
}
}
}
}
local hq = AICompany.GetCompanyHQ(Me);
xoilist.Valuate(AIIndustry.GetDistanceManhattanToTile, hq);
xoilist.Sort(AIList.SORT_BY_VALUE,true);
foreach (i, z in xoilist) {

local tiletown = AITile.GetClosestTown(AIIndustry.GetLocation(i));
if (AIIndustry.GetLastMonthProduction(i, cargo) > 0) { // remove industries which are already supplied
xoilist.RemoveItem(i);
} else if (!(townlist.HasItem(tiletown))) {

IndTownList.AddItem(tiletown, 0); // add this town to the list to connect even if it's not big enough
AILog.Info("But " + AIIndustry.GetName(i) + " is intriguing...")
return;
}
}
return;
}
}


if (go) {

// find the target and start tiles
// at source:

local a;
local b;
local adir;
local bdir;
local depot;

local rfolo = [AIMap.GetTileX(AIBaseStation.GetLocation(dest)),AIMap.GetTileY(AIBaseStation.GetLocation(dest))]
if (AIStation.GetStationID(AIMap.GetTileIndex(rfolo[0]+1, rfolo[1])) == dest) {
	if (AIRail.IsRailTile(AIMap.GetTileIndex(rfolo[0]+3, rfolo[1]))) {	
a = 	AIMap.GetTileIndex(rfolo[0]+6, rfolo[1]);
adir =  AIMap.GetTileIndex(rfolo[0]+5, rfolo[1]);
depot = AIMap.GetTileIndex(rfolo[0]+4, rfolo[1]-1);
	 } else {
a = 	AIMap.GetTileIndex(rfolo[0]-4, rfolo[1]);
adir =  AIMap.GetTileIndex(rfolo[0]-3, rfolo[1]);
depot = AIMap.GetTileIndex(rfolo[0]-2, rfolo[1]-1);
}
} else {
	if (AIRail.IsRailTile(AIMap.GetTileIndex(rfolo[0], rfolo[1]+3))) {	
a = 	AIMap.GetTileIndex(rfolo[0], rfolo[1]+6);
adir =  AIMap.GetTileIndex(rfolo[0], rfolo[1]+5);
depot = AIMap.GetTileIndex(rfolo[0]-1, rfolo[1]+4);
	 } else {
a = 	AIMap.GetTileIndex(rfolo[0], rfolo[1]-4);
adir =  AIMap.GetTileIndex(rfolo[0], rfolo[1]-3);
depot = AIMap.GetTileIndex(rfolo[0]-1, rfolo[1]-2);
}
}

rfolo = [AIMap.GetTileX(AIBaseStation.GetLocation(src)),AIMap.GetTileY(AIBaseStation.GetLocation(src))]
if (AIStation.GetStationID(AIMap.GetTileIndex(rfolo[0]+1, rfolo[1])) == src) {
	if (AIMap.GetTileX(AIBaseStation.GetLocation(dest)) > rfolo[0]) {
b = 	AIMap.GetTileIndex(rfolo[0]+3, rfolo[1]);
bdir =  AIMap.GetTileIndex(rfolo[0]+2, rfolo[1]);
	} else {
b = 	AIMap.GetTileIndex(rfolo[0]-1, rfolo[1]);
bdir =  AIMap.GetTileIndex(rfolo[0], rfolo[1]);
} 
}else {
	if (AIMap.GetTileY(AIBaseStation.GetLocation(dest)) > rfolo[1]) {
b = 	AIMap.GetTileIndex(rfolo[0], rfolo[1]+3);
bdir =  AIMap.GetTileIndex(rfolo[0], rfolo[1]+2);
	} else {
b = 	AIMap.GetTileIndex(rfolo[0], rfolo[1]-1);
bdir =  AIMap.GetTileIndex(rfolo[0], rfolo[1]);
} 
}

//AISign.BuildSign(a, "a");
//AISign.BuildSign(adir, "adir");
//AISign.BuildSign(b, "b");
//AISign.BuildSign(bdir, "bdir");

// override a and adir for the second line

if (secline) {
a = gotile;
adir = intile;
}

//build the line
if (BuildALine([[a, adir]], [[b, bdir]])) {


if (secline) {

// build connecting line and signals


while ((AIRail.GetRailTracks(contile) & conrail) != conrail) {
AIRail.BuildRailTrack(contile, conrail);
}

AIRail.BuildRail(contile, intile, gotile);

for(local n = 0; n < 3; n++){
		AIRail.BuildSignal(
		sigtile,
		sigtar[n],
		AIRail.SIGNALTYPE_PBS // how do we build semaphores, precious?
		);
		}

AIRail.BuildSignal(
		intile,
		gotile,
		AIRail.SIGNALTYPE_PBS // how do we build semaphores, precious?
		);
}


BuyATrain(dest, src, cargo, depot, false);

} else {
DudIndustries.AddItem(AIIndustry.GetIndustryID(suptile), 0); // don't try the same industry again
}
}


}

//===================================
//build a small station
//===================================

function CivilAI::SmallStation(ind, side, orientation, cargo) {

local xof;
local yof;
local indloc = AIIndustry.GetLocation(ind);

if (side == 0) {
xof = 0 // SE
yof = 4
} else if (side == 1) {
xof = 4 // SW
yof = 0
} else if (side == 2) {
xof = 0 // NW
yof = -3
} else {
xof = -3 // NE
yof = 0
}

// clockwise spiral out
local trytilegrid = [AIMap.GetTileX(indloc) + xof,AIMap.GetTileY(indloc) + yof] // we're making assumptions about industry size here
local trytile;
local x = 0
local y = 0
local i = 0
local s = 2
local NewStat = null;


while ((i < s) && NewStat == null) {
//y--
for (;y >= 0-i;y--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if ((AITileList_IndustryAccepting(ind, 4).HasItem(trytile))) {
if (NewStat = BuildSmallStation(ind, trytile, side, orientation, cargo)) {
return NewStat;
}
}
} 
//x--;
for (;x >= 0-i;x--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if ((AITileList_IndustryAccepting(ind, 4).HasItem(trytile))) {
if (NewStat = BuildSmallStation(ind, trytile, side, orientation, cargo)) {
return NewStat;
}
}
} 
//y++;
for (;y <= i;y++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if ((AITileList_IndustryAccepting(ind, 4).HasItem(trytile))) {
if (NewStat = BuildSmallStation(ind, trytile, side, orientation, cargo)) {
return NewStat;
}
}
} 
//x++;
for (;x <= i;x++) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if ((AITileList_IndustryAccepting(ind, 4).HasItem(trytile))) {
if (NewStat = BuildSmallStation(ind, trytile, side, orientation, cargo)) {
return NewStat;
} 
}
}
i=i+1
}

return NewStat;

}
//======
 function CivilAI::BuildSmallStation(ind, tile, side, orientation, cargo) {
 //=====
 local NewStat = null;
 local indloc = AIIndustry.GetLocation(ind); 
 
 //AISign.BuildSign(tile, side + ", " + orientation);
 
 // flatten land
local tg = [AIMap.GetTileX(tile),AIMap.GetTileY(tile)]

AITile.LevelTiles(indloc,tile);

if (side % 2 == 1) {
if (orientation == 0) {
	if (!(AITile.IsBuildableRectangle(AIMap.GetTileIndex(tg[0]-1,tg[1]), 3, 8))) { return null; }
AITile.LevelTiles(tile,AIMap.GetTileIndex(tg[0]+2,tg[1]+8));
AITile.LevelTiles(tile,AIMap.GetTileIndex(tg[0]-1,tg[1]+8));
} else {
 	if (!(AITile.IsBuildableRectangle(AIMap.GetTileIndex(tg[0]-1,tg[1]-6), 3, 8))) { return null; }
AITile.LevelTiles(tile,AIMap.GetTileIndex(tg[0]+2,tg[1]+3));
AITile.LevelTiles(tile,AIMap.GetTileIndex(tg[0]-1,tg[1]+3));
AITile.LevelTiles(tile,AIMap.GetTileIndex(tg[0]+2,tg[1]-6));
AITile.LevelTiles(tile,AIMap.GetTileIndex(tg[0]-1,tg[1]-6));
}
 } else {
 if (orientation == 0) {
 	if (!(AITile.IsBuildableRectangle(AIMap.GetTileIndex(tg[0],tg[1]-1), 8, 3))) { return null; }
AITile.LevelTiles(tile,AIMap.GetTileIndex(tg[0]+8,tg[1]+2));
AITile.LevelTiles(tile,AIMap.GetTileIndex(tg[0]+8,tg[1]-1));
} else {
 	if (!(AITile.IsBuildableRectangle(AIMap.GetTileIndex(tg[0]-6,tg[1]-1), 8, 3))) { return null; }
AITile.LevelTiles(tile,AIMap.GetTileIndex(tg[0]+3,tg[1]+2));
AITile.LevelTiles(tile,AIMap.GetTileIndex(tg[0]+3,tg[1]-1));
AITile.LevelTiles(tile,AIMap.GetTileIndex(tg[0]-6,tg[1]+2));
AITile.LevelTiles(tile,AIMap.GetTileIndex(tg[0]-6,tg[1]-1));
}
  }
  
if (side % 2 == 0) {
	if (orientation == 0) {
		// 0, 0 SE, S
		if (AIRail.BuildNewGRFRailStation(AIMap.GetTileIndex(tg[0],tg[1]),AIRail.RAILTRACK_NE_SW,1,3,AIStation.STATION_NEW,cargo,AIIndustryType.INDUSTRYTYPE_UNKNOWN, AIIndustry.GetIndustryType(ind), 64, false)) {
//		AILog.Info(cargo + " " + AIIndustry.GetIndustryType(ind));
		NewStat = (AIStation.GetStationID(AIMap.GetTileIndex(tg[0],tg[1])));
		AIRail.BuildRail(
		AIMap.GetTileIndex(tg[0]+2,tg[1]), //init tracks
		AIMap.GetTileIndex(tg[0]+3,tg[1]),
		AIMap.GetTileIndex(tg[0]+6,tg[1])
		);
		AIRail.BuildRailDepot(
		AIMap.GetTileIndex(tg[0]+4,tg[1]-1), //depot
		AIMap.GetTileIndex(tg[0]+4,tg[1])
		);
		// depot tracks
		AIRail.BuildRailTrack(
		AIMap.GetTileIndex(tg[0]+4,tg[1]),
		AIRail.RAILTRACK_NW_SW);
		AIRail.BuildRailTrack(
		AIMap.GetTileIndex(tg[0]+4,tg[1]),
		AIRail.RAILTRACK_NW_NE);
		// home signal
		AIRail.BuildSignal(
		AIMap.GetTileIndex(tg[0]+3,tg[1]),
		AIMap.GetTileIndex(tg[0]+2,tg[1]),
		AIRail.SIGNALTYPE_PBS // how do we build semaphores, precious?
		);
		return NewStat;
		} else {
		return null;
		}
	} else {
		// 0, 1 SE, N
		if (AIRail.BuildNewGRFRailStation(AIMap.GetTileIndex(tg[0],tg[1]),AIRail.RAILTRACK_NE_SW,1,3,AIStation.STATION_NEW,cargo,AIIndustryType.INDUSTRYTYPE_UNKNOWN, AIIndustry.GetIndustryType(ind), 64, false)) {
//		AILog.Info(cargo + " " + AIIndustry.GetIndustryType(ind));
		NewStat = (AIStation.GetStationID(AIMap.GetTileIndex(tg[0],tg[1])));
		AIRail.BuildRail(
		AIMap.GetTileIndex(tg[0],tg[1]), //init tracks
		AIMap.GetTileIndex(tg[0]-1,tg[1]),
		AIMap.GetTileIndex(tg[0]-4,tg[1])
		);
		AIRail.BuildRailDepot(
		AIMap.GetTileIndex(tg[0]-2,tg[1]-1), //depot
		AIMap.GetTileIndex(tg[0]-2,tg[1])
		);
		// depot tracks
		AIRail.BuildRailTrack(
		AIMap.GetTileIndex(tg[0]-2,tg[1]),
		AIRail.RAILTRACK_NW_SW);
		AIRail.BuildRailTrack(
		AIMap.GetTileIndex(tg[0]-2,tg[1]),
		AIRail.RAILTRACK_NW_NE);
		// home signal
		AIRail.BuildSignal(
		AIMap.GetTileIndex(tg[0]-1,tg[1]),
		AIMap.GetTileIndex(tg[0],tg[1]),
		AIRail.SIGNALTYPE_PBS // how do we build semaphores, precious?
		);
		return NewStat;
		} else {
		return null;
		}
	}  
} else {
	if (orientation == 0) {
		// 1, 0 SW, S
		if (AIRail.BuildNewGRFRailStation(AIMap.GetTileIndex(tg[0],tg[1]),AIRail.RAILTRACK_NW_SE,1,3,AIStation.STATION_NEW,cargo,AIIndustryType.INDUSTRYTYPE_UNKNOWN, AIIndustry.GetIndustryType(ind), 64, false)) {
//		AILog.Info(cargo + " " + AIIndustry.GetIndustryType(ind));
		NewStat = (AIStation.GetStationID(AIMap.GetTileIndex(tg[0],tg[1])));
		AIRail.BuildRail(
		AIMap.GetTileIndex(tg[0],tg[1]+2), //init tracks
		AIMap.GetTileIndex(tg[0],tg[1]+3),
		AIMap.GetTileIndex(tg[0],tg[1]+6)
		);
		AIRail.BuildRailDepot(
		AIMap.GetTileIndex(tg[0]-1,tg[1]+4), //depot
		AIMap.GetTileIndex(tg[0],tg[1]+4)
		);
		// depot tracks
		AIRail.BuildRailTrack(
		AIMap.GetTileIndex(tg[0],tg[1]+4),
		AIRail.RAILTRACK_NE_SE);
		AIRail.BuildRailTrack(
		AIMap.GetTileIndex(tg[0],tg[1]+4),
		AIRail.RAILTRACK_NW_NE);
		// home signal
		AIRail.BuildSignal(
		AIMap.GetTileIndex(tg[0],tg[1]+3),
		AIMap.GetTileIndex(tg[0],tg[1]+2),
		AIRail.SIGNALTYPE_PBS // how do we build semaphores, precious?
		);
		return NewStat;
		} else {
		return null;
		}
	} else {
		// 1, 1, SW, N
		if (AIRail.BuildNewGRFRailStation(AIMap.GetTileIndex(tg[0],tg[1]),AIRail.RAILTRACK_NW_SE,1,3,AIStation.STATION_NEW,cargo,AIIndustryType.INDUSTRYTYPE_UNKNOWN, AIIndustry.GetIndustryType(ind), 64, false)) {
//		AILog.Info(cargo + " " + AIIndustry.GetIndustryType(ind));
		NewStat = (AIStation.GetStationID(AIMap.GetTileIndex(tg[0],tg[1])));
		AIRail.BuildRail(
		AIMap.GetTileIndex(tg[0],tg[1]), //init tracks
		AIMap.GetTileIndex(tg[0],tg[1]-1),
		AIMap.GetTileIndex(tg[0],tg[1]-4)
		);
		AIRail.BuildRailDepot(
		AIMap.GetTileIndex(tg[0]-1,tg[1]-2), //depot
		AIMap.GetTileIndex(tg[0],tg[1]-2)
		);
		// depot tracks
		AIRail.BuildRailTrack(
		AIMap.GetTileIndex(tg[0],tg[1]-2),
		AIRail.RAILTRACK_NE_SE);
		AIRail.BuildRailTrack(
		AIMap.GetTileIndex(tg[0],tg[1]-2),
		AIRail.RAILTRACK_NW_NE);
		// home signal
		AIRail.BuildSignal(
		AIMap.GetTileIndex(tg[0],tg[1]-1),
		AIMap.GetTileIndex(tg[0],tg[1]),
		AIRail.SIGNALTYPE_PBS // how do we build semaphores, precious?
		);
		return NewStat;
		} else {
		return null;
		}
	}  
}
}

//==========
function CivilAI::TrainPickup(source, ind, cargo) {
//===========

// clockwise spiral out
local trytilegrid = [AIMap.GetTileX(source),AIMap.GetTileY(source)] // we're making assumptions about industry size here
local trytile;
local x = 0
local y = 0
local i = 0
local s = 8
local NewStat = null;
local gotile;


while ((i < s) && NewStat == null) {
//y--
for (;y >= 0-i;y--) {
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
//AISign.BuildSign(trytile, ".");
if (AITile.IsBuildableRectangle(trytile, 7,2)) {
gotile = AIMap.GetTileIndex(AIMap.GetTileX(trytile)+ 2,AIMap.GetTileY(trytile) );
	if (AITileList_IndustryProducing(AIIndustry.GetIndustryID(source),4).HasItem(gotile)) {
	if (AIRail.BuildNewGRFRailStation(gotile,AIRail.RAILTRACK_NE_SW,1,3,AIStation.STATION_NEW,cargo,AIIndustry.GetIndustryType(ind), AIIndustryType.INDUSTRYTYPE_UNKNOWN, 64, true)) {
				NewStat = (AIStation.GetStationID(gotile));
				
				//AISign.BuildSign(trytile, "T Y-");
				//AISign.BuildSign(gotile, "G");
				
	return NewStat;
}
}
}
} 
//x--;
for (;x >= 0-i;x--) {
//AISign.BuildSign(trytile, ".");
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
if (AITile.IsBuildableRectangle(trytile, 2,7)) {
gotile = AIMap.GetTileIndex(AIMap.GetTileX(trytile) ,AIMap.GetTileY(trytile)+ 2);
	if (AITileList_IndustryProducing(AIIndustry.GetIndustryID(source),4).HasItem(gotile)) {
	if (AIRail.BuildNewGRFRailStation(gotile,AIRail.RAILTRACK_NW_SE,1,3,AIStation.STATION_NEW,cargo,AIIndustry.GetIndustryType(ind), AIIndustryType.INDUSTRYTYPE_UNKNOWN, 64, true)) {
			NewStat = (AIStation.GetStationID(gotile));
			
				//AISign.BuildSign(trytile, "T X-");
				//AISign.BuildSign(gotile, "G");
	return NewStat;
}
}
}
} 
//y++;
for (;y <= i;y++) {
//AISign.BuildSign(trytile, ".");
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
if (AITile.IsBuildableRectangle(trytile, 7,2)) {
gotile = AIMap.GetTileIndex(AIMap.GetTileX(trytile)+ 2,AIMap.GetTileY(trytile) );
	if (AITileList_IndustryProducing(AIIndustry.GetIndustryID(source),4).HasItem(gotile)) {
	if (AIRail.BuildNewGRFRailStation(gotile,AIRail.RAILTRACK_NE_SW,1,3,AIStation.STATION_NEW,cargo,AIIndustry.GetIndustryType(ind), AIIndustryType.INDUSTRYTYPE_UNKNOWN, 64, true)) {
				NewStat = (AIStation.GetStationID(gotile));
				
				//AISign.BuildSign(trytile, "T Y+");
				//AISign.BuildSign(gotile, "G");	
	return NewStat;
}
}
}
} 
//x++;
for (;x <= i;x++) {
//AISign.BuildSign(trytile, ".");
trytile = AIMap.GetTileIndex(trytilegrid[0]+x,trytilegrid[1]+y);
if (AITile.IsBuildableRectangle(trytile, 2,7)) {
gotile = AIMap.GetTileIndex(AIMap.GetTileX(trytile) ,AIMap.GetTileY(trytile)+ 2);
	if (AITileList_IndustryProducing(AIIndustry.GetIndustryID(source),4).HasItem(gotile)) {
	if (AIRail.BuildNewGRFRailStation(gotile,AIRail.RAILTRACK_NW_SE,1,3,AIStation.STATION_NEW,cargo,AIIndustry.GetIndustryType(ind), AIIndustryType.INDUSTRYTYPE_UNKNOWN, 64, true)) {
				NewStat = (AIStation.GetStationID(gotile));
				
				//AISign.BuildSign(trytile, "T X+");
				//AISign.BuildSign(gotile, "G");				
	return NewStat;
}
}
}
} 
i=i+1
}

return NewStat;
}

// find the train depot at this station =========
 function CivilAI::FindTrainDepot(station) {
 
local depot;
local rfolo = [AIMap.GetTileX(AIBaseStation.GetLocation(station)),AIMap.GetTileY(AIBaseStation.GetLocation(station))]
if (AIStation.GetStationID(AIMap.GetTileIndex(rfolo[0]+1, rfolo[1])) == station) {
	if (AIRail.IsRailTile(AIMap.GetTileIndex(rfolo[0]+3, rfolo[1]))) {	
depot = AIMap.GetTileIndex(rfolo[0]+4, rfolo[1]-1);
	 } else {
depot = AIMap.GetTileIndex(rfolo[0]-2, rfolo[1]-1);
}
} else {
	if (AIRail.IsRailTile(AIMap.GetTileIndex(rfolo[0], rfolo[1]+3))) {	
depot = AIMap.GetTileIndex(rfolo[0]-1, rfolo[1]+4);
	 } else {
depot = AIMap.GetTileIndex(rfolo[0]-1, rfolo[1]-2);
}
}

return depot;

}


// ==========================================
// Buy a Train! Choo Choo!
// ==========================================
 function CivilAI::BuyATrain(a, b, cargo, depot, costing) {
 
 // first, pick a loco
 
local loco;
local train;
local dosh;
 
local locolist = AIEngineList(AIVehicle.VT_RAIL);
 foreach (l, z in locolist) {
//AILog.Info("Assessing " + AIEngine.GetName(l));
 
 if (
 AIEngine.GetCapacity(l) > 0
 || AIEngine.GetPower(l) < 250 
 || AIEngine.GetMaxSpeed(l) < 64 // these are our minimum standards 
 || !AIEngine.CanRunOnRail(l, 0)
 || !AIEngine.HasPowerOnRail(l, 0)
 )
 {locolist.RemoveItem(l)} // no wogans or MUs or electrics
 }
 locolist.Valuate(AIEngine.GetRunningCost);
 locolist.Sort(AIList.SORT_BY_VALUE, true);
  
if (locolist.Count() == 0) {
AILog.Info("I couldn't find a suitable locomotive.");
dosh = 1000000000;
return (dosh);
} else {
loco = locolist.Begin();
dosh = AIEngine.GetPrice(loco);
if (!costing) {
AILog.Info("I picked " + AIEngine.GetName(loco) + " as our locomotive.");
train = AIVehicle.BuildVehicle(depot, loco);
}

// pick wogans

local woganlist = AIEngineList(AIVehicle.VT_RAIL);
foreach (w, z in woganlist) {
//AILog.Info("Assessing " + AIEngine.GetName(w));
 
 if (
 AIEngine.GetCapacity(w) == 0
 || AIEngine.GetPower(w) > 0 
 || (AIEngine.GetMaxSpeed(w) < 64 && AIEngine.GetMaxSpeed(w) != 0) // these are our minimum standards  
 || !AIEngine.CanRunOnRail(w, 0)
 || !AIEngine.CanRefitCargo(w, cargo)
 )
 {woganlist.RemoveItem(w)} // no locos, unrefits or non-rail vehicles
 }
 if (woganlist.Count() == 0) {
AILog.Info("I couldn't find any suitable wagons.");
dosh = 1000000000;
return (dosh);
} else {

local unit;
local randomwogans;
local newwogan;
local unitlist = AIList();
unitlist.AddList(woganlist);

 if (
 (AICargo.HasCargoClass(cargo, AICargo.CC_BULK)
 || AICargo.HasCargoClass(cargo, AICargo.CC_LIQUID))
 && !AICargo.HasCargoClass(cargo, AICargo.CC_PIECE_GOODS)
 ) {
  // unit train for bulk, liquid
  
  // prefer native vehicles
 foreach (w, z in unitlist) {
 if (AIEngine.GetCargoType(w) != cargo) {
 unitlist.RemoveItem(w);
 }
 } 
 if (unitlist.Count() > 0) {
  unitlist.Valuate(AIEngine.GetCapacity);
  unitlist.Sort(AIList.SORT_BY_VALUE, false);
  unit = unitlist.Begin();
 } else {
  woganlist.Valuate(AIEngine.GetCapacity);
  woganlist.Sort(AIList.SORT_BY_VALUE, false);
  unit = woganlist.Begin();
  }  
 } else {
 // randomwogans for piece goods
 randomwogans = true;
 woganlist.Valuate(AIEngine.GetPrice); // choose most expensive for costing
 unit = woganlist.Begin();
  }
  
  if (costing) {
  dosh = dosh + (AIEngine.GetPrice(unit) * 5);
  return (dosh);  
  }

while (AIVehicle.GetLength(train) < (16 * 2.5) + 1) {
if (randomwogans) {
 woganlist.Valuate(AIBase.RandItem); // shuffle the list
 unit = woganlist.Begin();
}
  
  newwogan = AIVehicle.BuildVehicle(depot, unit);
  AIVehicle.RefitVehicle(newwogan, cargo);  
  AIVehicle.MoveWagon(newwogan, 0, train, AIVehicle.GetNumWagons(train)-1);
}

 
// give the orders and start the train!

AIOrder.AppendOrder(train,AIBaseStation.GetLocation(b),(AIOrder.OF_NON_STOP_INTERMEDIATE + AIOrder.OF_FULL_LOAD_ANY));
AIOrder.AppendOrder(train,AIBaseStation.GetLocation(a),(AIOrder.OF_NON_STOP_INTERMEDIATE + AIOrder.OF_NO_LOAD));

local trainnames = [
"Monty", "Horace", "Apollo", "Mercury", "Vulcan", 
"Cupid", "Sooty", "George", "Comet", "Ajax", 
AICompany.GetPresidentName(Me), "Hercules", "Amazon", "Atlas", "Rocket"]

local nx = 0
local name = null
local changename = false
local trainname = trainnames[AIBase.RandRange(15)]

while (!changename) {
nx++ 
if (nx < 2) {
name = (trainname)
} else {
name = (trainname + " " + nx)
}
changename = AIVehicle.SetName(train,name)
}

AIVehicle.StartStopVehicle(train); // choo choo!
}
}
}

function CivilAI::RemoveTracksFrom(station){


local rfolo = [AIMap.GetTileX(AIBaseStation.GetLocation(station)),AIMap.GetTileY(AIBaseStation.GetLocation(station))]
local t;
local t2;
local lastdir; // where we came from, not where we're heading

// first, establish which end of the station we leave

if (AIRail.GetRailStationDirection(AIBaseStation.GetLocation(station)) == AIRail.RAILTRACK_NE_SW) {

t = AIMap.GetTileIndex(rfolo[0]-1, rfolo[1]);
t2 = AIMap.GetTileIndex(rfolo[0]+3, rfolo[1]);

if (
AIRail.GetRailTracks(t) == AIRail.RAILTRACK_NE_SW ||
AIRail.GetRailTracks(t) == AIRail.RAILTRACK_NW_SW ||
AIRail.GetRailTracks(t) == AIRail.RAILTRACK_SW_SE
) {

lastdir = 1;

} else {
lastdir = 3;
t = t2;
}
} else {
t = AIMap.GetTileIndex(rfolo[0], rfolo[1]-1);
t2 = AIMap.GetTileIndex(rfolo[0], rfolo[1]+3);

if (
AIRail.GetRailTracks(t) == AIRail.RAILTRACK_NW_SE ||
AIRail.GetRailTracks(t) == AIRail.RAILTRACK_NE_SE ||
AIRail.GetRailTracks(t) == AIRail.RAILTRACK_SW_SE
) {
lastdir = 0;
} else {
lastdir = 2;
t = t2;
}
}

rfolo = [AIMap.GetTileX(t),AIMap.GetTileY(t)] // reset rfolo

//AILog.Info(lastdir);

while (true) { // careful now

//AISign.BuildSign(t, "..")

if (AITunnel.IsTunnelTile(t)) { t = AITunnel.GetOtherTunnelEnd(t); rfolo = [AIMap.GetTileX(t),AIMap.GetTileY(t)];} 
else if (AIBridge.IsBridgeTile(t)) {t = AIBridge.GetOtherBridgeEnd(t); rfolo = [AIMap.GetTileX(t),AIMap.GetTileY(t)];} // that was easy enough
else {

if (lastdir == 0) {
	if (AIRail.GetRailTracks(t) == AIRail.RAILTRACK_NW_SE) {lastdir = 0} 
else if (AIRail.GetRailTracks(t) == AIRail.RAILTRACK_NE_SE) {lastdir = 1} 
else if (AIRail.GetRailTracks(t) == AIRail.RAILTRACK_SW_SE) {lastdir = 3} 
else {AILog.Info("Finished removing tracks"); return;}
} else if (lastdir == 1) {
	if (AIRail.GetRailTracks(t) == AIRail.RAILTRACK_NE_SW) {lastdir = 1} 
else if (AIRail.GetRailTracks(t) == AIRail.RAILTRACK_NW_SW) {lastdir = 0} 
else if (AIRail.GetRailTracks(t) == AIRail.RAILTRACK_SW_SE) {lastdir = 2} 
else {AILog.Info("Finished removing tracks"); return;}
} else if (lastdir == 2) {
	if (AIRail.GetRailTracks(t) == AIRail.RAILTRACK_NW_SE) {lastdir = 2} 
else if (AIRail.GetRailTracks(t) == AIRail.RAILTRACK_NW_NE) {lastdir = 1} 
else if (AIRail.GetRailTracks(t) == AIRail.RAILTRACK_NW_SW) {lastdir = 3} 
else {AILog.Info("Finished removing tracks"); return;}
} else {
	if (AIRail.GetRailTracks(t) == AIRail.RAILTRACK_NE_SW) {lastdir = 3} 
else if (AIRail.GetRailTracks(t) == AIRail.RAILTRACK_NW_NE) {lastdir = 0} 
else if (AIRail.GetRailTracks(t) == AIRail.RAILTRACK_NE_SE) {lastdir = 2} 
else {AILog.Info("Finished removing tracks"); return;}
}
}

if (AITile.HasTransportType(t, AITile.TRANSPORT_ROAD)) {

AIRail.RemoveRailTrack(t, AIRail.RAILTRACK_NW_SE);
AIRail.RemoveRailTrack(t, AIRail.RAILTRACK_NE_SW);

} else {
AITile.DemolishTile(t);
}

if (lastdir == 0) {rfolo[1]--;}
else if (lastdir == 1) {rfolo[0]--;}
else if (lastdir == 2) {rfolo[1]++;}
else {rfolo[0]++;}

t = AIMap.GetTileIndex(rfolo[0], rfolo[1]);
}





}

// ====================================================== 
//                  BUILD RAIL CONNECTION
// ====================================================== 

function CivilAI::BuildALine(a,b) {


AILog.Info("I'm building a rail line!")


local buildrail = RailPF();
buildrail._max_cost = 10000000;
buildrail._cost_tile = 100;
buildrail._cost_diagonal_tile = 100; // 70;
buildrail._cost_turn = 40; // 50;
buildrail._cost_slope = 300; // 100;
buildrail._cost_bridge_per_tile = 150;
buildrail._cost_tunnel_per_tile = 110; // 120;
buildrail._cost_coast = 20;
buildrail._max_bridge_length = 8 // 6;
buildrail._max_tunnel_length = 16; // 6;

buildrail.InitializePath(a, b);

local path = false;
local i = 0;
while (path == false) {
  path = buildrail.FindPath(20);
  AIController.Sleep(1);
//AILog.Info(i)
   i++
   
   if (i > TrainRange * 10) {
   AILog.Info("I couldn't find a path.")
   return false;
   }
   
   
}


local prev = null;
local prevprev = null;
while (path != null) {
  if (prevprev != null) {
    if (AIMap.DistanceManhattan(prev, path.GetTile()) > 1) {
      if (AITunnel.GetOtherTunnelEnd(prev) == path.GetTile()) {
        AITunnel.BuildTunnel(AIVehicle.VT_RAIL, prev);
      } else {
        local bridge_list = AIBridgeList_Length(AIMap.DistanceManhattan(path.GetTile(), prev) + 1);
        bridge_list.Valuate(AIBridge.GetMaxSpeed);
        bridge_list.Sort(AIList.SORT_BY_VALUE, false);
        AIBridge.BuildBridge(AIVehicle.VT_RAIL, bridge_list.Begin(), prev, path.GetTile());
      }
      prevprev = prev;
      prev = path.GetTile();
      path = path.GetParent();
    } else {
      if (AIRail.BuildRail(prevprev, prev, path.GetTile())) {
	// AILog.Info("All is fine")
	  } else {
	  AILog.Info("Something went wrong!")
	  return false;
	  }
    }
  }
  if (path != null) {
    prevprev = prev;
    prev = path.GetTile();
    path = path.GetParent();
  }
}


//AILog.Info("I've finished rail building for now.")
return true;
}



 // That's all folks.






